#!/usr/bin/env python

import click
from elements.cli.lib.aliased_group import AliasedGroup
from elements.cli.commands.algorithm import algorithm
from elements.cli.commands.analysis import analysis
from elements.cli.commands.aoi import aoi
from elements.cli.commands.environment import environment
from elements.cli.commands.permission import permission
from elements.cli.commands.credit import credit
from elements.cli.commands.toi import toi
from elements.cli.commands.data import data
from elements.cli.commands.imagery import imagery
from elements.cli.commands.visualization import visualization
from elements.cli.commands.manifest import manifest
from elements.cli.commands.tasks import tasks


@click.command(cls=AliasedGroup)
@click.pass_context
def main(ctx):
    pass


main.add_command(algorithm)
main.add_command(analysis)
main.add_command(aoi)
main.add_command(environment)
main.add_command(permission)
main.add_command(credit)
main.add_command(toi)
main.add_command(data)
main.add_command(imagery)
main.add_command(visualization)
main.add_command(manifest)
main.add_command(tasks)


if __name__ == '__main__':
    main(obj={})



from google.protobuf.struct_pb2 import Struct

from elements_api import ElementsAsyncClient
from elements_api.models.filter_pb2 import (
    Filter, FilterCreateRequest, FilterDeleteRequest, FilterListRequest, FilterMappingCreateRequest
)
from elements_api.models.project_filter_pb2 import (
    ProjectFilter, ProjectFilterMappingCreateRequest
)
from elements.sdk.tools.sdk_support import SDKSupport

class APIFilter:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client
        self.sdk_support = SDKSupport()

    async def create(self, expression: str, filter_language: str, data_type: str = None, name: str = None,
                     description: str = None, metadata: Struct = None) -> str:
        request = FilterCreateRequest(
            expression=expression,
            data_type=data_type,
            name=name,
            description=description,
            filter_language=filter_language,
            metadata=metadata
        )
        response = await self.__client.api.filter.create(request)
        return response.filter_id

    async def delete(self, filter_ids: list[str]) -> None:
        request = FilterDeleteRequest(
            ids=filter_ids,
        )
        await self.__client.api.filter.delete(request)

    async def list(self, data_types: list[str], search_text: str) -> list[Filter]:
        request = FilterListRequest(
            data_types=data_types,
            search_text=search_text
        )
        responses = await self.sdk_support.get_all_paginated_objects(request,
                                                                     api_function=self.__client.api.filter.list,
                                                                     timeout=self.__timeout)
        filters = []
        for response in responses:
            filters.extend(response.filters)
        return filters

    async def create_mapping(self, computation_id: str, filter_id: str) -> None:
        request = FilterMappingCreateRequest(
            computation_id=computation_id,
            filter_id=filter_id
        )
        await self.__client.api.filter.create_mapping(request)


class APIProjectFilter:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client
        self.sdk_support = SDKSupport()

    async def create_mapping(self, filter_id: str, analysis_config_id: str, node_name: str, project_id: str,
                             input_filter: bool) -> None:
        request = ProjectFilterMappingCreateRequest(
            project_filters=[
                ProjectFilter(
                    filter=Filter(id=filter_id),
                    analysis_config_id=analysis_config_id,
                    node_name=node_name,
                    project_id=project_id,
                    input_filter=input_filter
                )
            ]
        )
        await self.__client.api.project_filter.create_mapping(request)



# flake8: noqa: E501
import os
import re
import logging
import tempfile
from zipfile import ZipFile
from collections import defaultdict
from typing import AsyncIterator, Dict, List, Optional

import boto3
import httpx
from google.protobuf.timestamp_pb2 import Timestamp

from elements_api import ElementsAsyncClient
from elements_api.models.common_models_pb2 import Pagination
from elements_api.models.result_pb2 import ResultGetRequest, ResultGetResponse


class APIResult:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client
        # External HTTP endpoints (overridable via environment)
        self.__results_api_base_url = os.getenv(
            "RESULT_DATA_SOURCE_API_URL",
            "https://results.orbitalinsight.com"
        )
        self.__app_api_base_url = os.getenv(
            "ELEMENTS_API_URL",
            "https://terrascope-app.orbitalinsight.com/api/v1"
        )
        self.__api_token = os.getenv("ELEMENTS_API_TOKEN")
        self.__httpx_timeout = httpx.Timeout(
            float(self.__timeout), read=300.0
        )

    @staticmethod
    async def merge_download_files(
        algorithm_computation_id_to_data_type_to_downloaded_paths: 
            Dict[str, Dict[str, List[str]]],
        download_dir: str = None
    ) -> Dict[str, Dict[str, str]]:
        download_dir = os.getcwd() if not download_dir else download_dir
        if not os.path.exists(download_dir):
            os.makedirs(download_dir)

        algorithm_computation_id_to_data_type_to_merged_file =             defaultdict(lambda: defaultdict(str))
        for (algorithm_computation_id,
                data_type_to_downloaded_paths) in algorithm_computation_id_to_data_type_to_downloaded_paths.items():
            for data_type, downloaded_paths in data_type_to_downloaded_paths.items():
                merged_file_dir = os.path.join(download_dir, algorithm_computation_id)
                os.makedirs(merged_file_dir, exist_ok=True)
                merged_file = os.path.join(merged_file_dir, f'{data_type}.csv')
                logging.info(
                    f"Merging files for algorithm_computation_id {algorithm_computation_id} "
                    f"and data_type {data_type} to {merged_file}"
                )

                header_written = False
                with open(merged_file, 'w') as output_csv:
                    for idx, downloaded_path in enumerate(downloaded_paths):
                        with tempfile.TemporaryDirectory() as working_dir:
                            interim_path = f'{working_dir}/{idx}'
                            os.mkdir(interim_path)
                            with ZipFile(downloaded_path, 'r') as zip_ref:
                                zip_ref.extractall(interim_path)
                            csv_path = f'{interim_path}/{data_type}.csv'
                            with open(csv_path, 'r') as input_csv:
                                if header_written:
                                    next(input_csv)
                                output_csv.write(input_csv.read())
                                header_written = True
                algorithm_computation_id_to_data_type_to_merged_file[algorithm_computation_id][data_type] = merged_file
        return algorithm_computation_id_to_data_type_to_merged_file

    async def download(
        self, algorithm_computation_ids: List[str] = None,
        analysis_computation_ids: List[str] = None,
        source_aoi_version: int = None, dest_aoi_version: int = None,
        algo_config_class: str = None,
        algo_config_subclass: str = None, created_on: Timestamp = None,
        observation_start_ts: Timestamp = None,
        max_observation_start_ts: Timestamp = None, data_type: str = None,
        download_dir: str = None
    ) -> Dict[str, Dict[str, List[str]]]:
        """
        algorithm_computation_ids: [Required] List[str] - Algorithm computation IDs

        :return: Dict[str, Dict[str, List[str]]]: mapping of algorithm_computation_id_to_data_type_to_downloaded_paths
        """
        download_dir = os.getcwd() if not download_dir else download_dir

        result_get_responses = await self.get(
            algorithm_computation_ids=algorithm_computation_ids,
            analysis_computation_ids=analysis_computation_ids,
            source_aoi_version=source_aoi_version,
            dest_aoi_version=dest_aoi_version,
            algo_config_class=algo_config_class,
            algo_config_subclass=algo_config_subclass,
            created_on=created_on,
            observation_start_ts=observation_start_ts,
            max_observation_start_ts=max_observation_start_ts,
            include_export_files=True, data_type=data_type
        )

        result_export_credentials = result_get_responses[0].export_credentials
        credentials = result_export_credentials.credentials
        s3 = boto3.client(
            's3',
            aws_access_key_id=credentials.fields['AccessKeyId'].string_value,
            aws_session_token=credentials.fields['SessionToken'].string_value,
            aws_secret_access_key=credentials.fields['SecretAccessKey'].string_value
        )
        pattern = r"https://(.*?)\.s3"
        container_name = re.search(pattern, result_export_credentials.base_url_template).group(1)
        algorithm_computation_id_to_data_type_to_downloaded_paths =             defaultdict(lambda: defaultdict(list))

        for result_get_response in result_get_responses:
            for result in result_get_response.results:
                data_type = result.data_type
                algorithm_computation_id = result.algorithm_computation_id
                for observation in result.observations:
                    key_path = observation.export_file.url
                    if not key_path:
                        continue
                    full_download_path = download_dir + os.path.split(key_path)[0]
                    filename = 'results.zip'
                    os.makedirs(full_download_path)
                    downloaded_path = os.path.join(full_download_path, filename)
                    s3.download_file(container_name, key_path[1:], downloaded_path)
                    algorithm_computation_id_to_data_type_to_downloaded_paths[
                        algorithm_computation_id][data_type].append(downloaded_path)

        logging.info(
            f"Downloaded results for algorithm_computation_ids and data_types: "
            f"{algorithm_computation_id_to_data_type_to_downloaded_paths}"
        )
        return algorithm_computation_id_to_data_type_to_downloaded_paths

    async def get(
        self, algorithm_computation_ids: List[str] = None,
        analysis_computation_ids: List[str] = None,
        source_aoi_version: int = None, dest_aoi_version: int = None,
        algo_config_class: str = None,
        algo_config_subclass: str = None, created_on: Timestamp = None,
        observation_start_ts: Timestamp = None,
        max_observation_start_ts: Timestamp = None,
        include_export_files: bool = None, data_type: str = None
    ) -> List[ResultGetResponse]:
        """
            required: algorithm_computation_ids or analysis_computation_ids
        """
        # Query all GetResultResponses
        result_get_responses = []
        pagination = Pagination(page_size=1000)
        has_next_result = True
        while has_next_result:
            request = ResultGetRequest(
                algorithm_computation_ids=algorithm_computation_ids,
                analysis_computation_ids=analysis_computation_ids,
                source_aoi_version=source_aoi_version,
                dest_aoi_version=dest_aoi_version,
                algo_config_class=algo_config_class,
                algo_config_subclass=algo_config_subclass,
                created_on=created_on,
                observation_start_ts=observation_start_ts,
                max_observation_start_ts=max_observation_start_ts,
                include_export_files=include_export_files,
                data_type=data_type,
                pagination=Pagination(
                    page_token=pagination.next_page_token, page_size=1000
                )
            )
            result_get_response = await self.__client.api.result.get(request, timeout=self.__timeout)
            result_get_responses.append(result_get_response)
            pagination = result_get_response.pagination
            has_next_result = pagination and pagination.next_page_token
        return result_get_responses

    # ---------------------------
    # Results Data Source methods
    # ---------------------------
    async def list_available_collections(self, project_id: str) -> List[str]:
        """
        List available collection IDs for a Terrascope project via the app API.

        Returns a list of strings formatted as "<algorithm_computation_id>---<data_type>".
        """
        if not self.__api_token:
            logging.error("ELEMENTS_API_TOKEN not set; cannot list collections")
            return []
        url = f"{self.__app_api_base_url}/projects/{project_id}/summary"
        try:
            async with httpx.AsyncClient(timeout=self.__httpx_timeout) as client:
                r = await client.get(url, headers=self.get_headers())
                if r.status_code != 200:
                    logging.error(
                        "Failed to fetch project summary for %s: %s %s",
                        project_id,
                        r.status_code,
                        r.text,
                    )
                    return []
                summary_info = r.json()
        except Exception as e:
            logging.exception("Exception fetching project summary for %s: %s", project_id, e)
            return []

        collection_ids: List[str] = []
        analysis_info = summary_info.get(
            "data", {}
        ).get("analysis_computation_info", {})
        for _, computations in analysis_info.items():
            for computation_info in computations:
                computation_id = computation_info.get("algorithm_computation_id")
                output_types = computation_info.get(
                    "algorithm_config_details", {}
                ).get("output_data_types", [])
                for data_type in output_types:
                    collection_ids.append(f"{computation_id}---{data_type}")
        return collection_ids

    async def iter_arrow_batches(
        self,
        collection_id: str,
        start_time: Optional[object] = None,
        end_time: Optional[object] = None,
        params: dict = None,
    ) -> AsyncIterator[object]:
        """
        Stream Arrow RecordBatches for a given collection_id from the Results Data Source API.

        Optional time filtering can be specified using RFC 3339 datetimes:
        - start_time and end_time may be strings already formatted per RFC 3339 or
          Python datetime instances. If only one bound is provided, an open range is used
          for the other bound as per RFC 3339 section 5.6 (".." for open).
        """
        try:
            import pyarrow.ipc as ipc  # type: ignore
        except ImportError as e:
            raise ImportError(
                "pyarrow is required for iter_arrow_batches; install elements-sdk with pyarrow."
            ) from e

        if not self.__api_token:
            raise ValueError("ELEMENTS_API_TOKEN not set; cannot fetch arrow stream")

        feature_url = f"{self.__results_api_base_url}/features/{collection_id}"

        # Build optional datetime range query param
        def _to_rfc3339(value: object) -> str:
            from datetime import datetime, timezone
            if isinstance(value, str):
                return value
            if isinstance(value, datetime):
                # If naive, assume UTC. If tz-aware, convert to UTC and use Z suffix
                if value.tzinfo is None:
                    return value.replace(tzinfo=timezone.utc).isoformat().replace("+00:00", "Z")
                return value.astimezone(timezone.utc).isoformat().replace("+00:00", "Z")
            raise TypeError("start_time/end_time must be str or datetime if provided")

        params = params or {}
        if start_time is not None and end_time is not None:
            start_str = _to_rfc3339(start_time)
            end_str = _to_rfc3339(end_time)
            params["datetime"] = f"{start_str}/{end_str}"

        async with httpx.AsyncClient(timeout=self.__httpx_timeout) as client:
            try:
                async with client.stream("GET", feature_url, headers=self.get_headers(), params=params) as response:
                    if response.status_code != 200:
                        body = await response.aread()
                        logging.error(
                            "Failed to get Arrow stream for %s: %s, body: %s",
                            collection_id,
                            response.status_code,
                            body.decode(errors="ignore"),
                        )
                        return
                    with tempfile.SpooledTemporaryFile() as tmp:
                        async for chunk in response.aiter_bytes():
                            tmp.write(chunk)
                        if tmp.tell() == 0:
                            return
                        tmp.seek(0)
                        try:
                            with ipc.open_stream(tmp) as reader:
                                for batch in reader:
                                    yield batch
                        except Exception:
                            logging.exception("Failed to parse Arrow stream for %s", collection_id)
                            return
            except Exception:
                logging.exception("Exception fetching Arrow stream for %s", collection_id)
                return

    async def download_parquet(
            self, collection_id: str, dest_path: str, start_time: Optional[object] = None,
            end_time: Optional[object] = None, params: dict = None) -> int:
        """
        Download a collection as Parquet to dest_path. Returns total rows written.
        """
        try:
            import pyarrow.parquet as pq  # type: ignore
        except ImportError as e:
            raise ImportError(
                "pyarrow is required for download_parquet; install elements-sdk with pyarrow."
            ) from e

        total_rows = 0
        writer = None
        try:
            async for batch in self.iter_arrow_batches(
                    collection_id, start_time=start_time, end_time=end_time, params=params):
                if batch is None:
                    continue
                if writer is None:
                    writer = pq.ParquetWriter(dest_path, batch.schema)  # type: ignore
                writer.write_batch(batch)
                total_rows += batch.num_rows
        finally:
            try:
                if writer is not None:
                    writer.close()  # type: ignore
            except Exception:
                pass
        return total_rows

    def get_headers(self):
        _ = self.__client.api.refresh_access_token()
        access_token = self.__client.api.access_token
        return {"Authorization": f"Bearer {access_token}"}

    async def get_collection_info(self, collection_id: str) -> Optional[dict]:
        """
        Retrieve collection info metadata from the Results Data Source API.
        """
        if not self.__api_token:
            logging.error("ELEMENTS_API_TOKEN not set; cannot get collection info")
            return None
        url = f"{self.__results_api_base_url}/collections/{collection_id}"
        try:
            async with httpx.AsyncClient(timeout=self.__httpx_timeout) as client:
                response = await client.get(url, headers=self.get_headers())
                if response.status_code != 200:
                    logging.error(
                        "Failed to get collection info for %s: %s %s",
                        collection_id,
                        response.status_code,
                        await response.aread(),
                    )
                    return None
                return response.json()
        except Exception:
            logging.exception("Exception fetching collection info for %s", collection_id)
            return None



#!/usr/bin/env python

import click
import asyncio
import os
import elements.cli.lib.workflow as wf
import elements.cli.lib.utils as tsu
from elements.cli.lib.aliased_group import AliasedGroup


@click.command(cls=AliasedGroup, help="Analysis 'computation' command group")
@click.pass_context
def computation(ctx):
    pass


@computation.command('get')
@click.pass_context
@click.option('-iP', '--analysis_computation_id', type=str)
def analysis_computation_get(ctx, analysis_computation_id):

    wf.check_environment_complete(raise_on_failure=True)

    analysis_computation_infos = asyncio.run(
        wf.get_analysis_computation_info()
    )

    for analysis_computation_info in analysis_computation_infos:
        info = tsu.protobuf_to_dict(analysis_computation_info)

        # if they asked for a specific one, just print that.  otherwise print all.
        if (analysis_computation_id is None) or \
           (analysis_computation_id and analysis_computation_id == info['id']):
            click.secho(f"analysis_computation_id: {info['id']}", fg='cyan')
            for k, v in info.items():
                click.echo(f"--> {k:20s} {v}")


@computation.command('create')
@click.pass_context
@click.option('-iC', '--analysis_config_id', type=str)
@click.option('--dry_run', is_flag=True)
@click.option('-v', '--verbose', is_flag=True)
@click.option('-it', '--toi_id', type=str)
@click.option('-ia', '--aoi_collection_id', type=str)
def analysis_computation_create(
        ctx,
        analysis_config_id,
        dry_run,
        verbose,
        toi_id,
        aoi_collection_id,
):

    wf.check_environment_complete(raise_on_failure=True)

    click.echo("Using the following information:")
    click.echo(f"--> analysis_config_id:       {analysis_config_id}")
    click.echo(f"--> toi_id:                   {toi_id}")
    click.echo(f"--> aoi_collection_id:        {aoi_collection_id}")

    if not dry_run:

        computation_id = asyncio.run(
            wf.create_analysis_computation(
                analysis_config_id,
                toi_id,
                aoi_collection_id,
            )
        )
        click.echo(f"computation_id: {computation_id}")


@computation.command('create_scratch')
@click.pass_context
@click.option('-iC', '--analysis_config_id', type=str)
@click.option('-s', '--start_date', type=str)
@click.option('-e', '--end_date', type=str)
@click.option('-af', '--aoi_file', type=str)
@click.option('-d', '--date_format', type=str, default='%Y-%m-%d')
@click.option('-f', '--frequency', type=click.IntRange(min=2, max=3, clamp=False), default=3,
              help="HOURLY=2, DAILY=3")
@click.option('--dry_run', is_flag=True)
@click.option('-v', '--verbose', is_flag=True)
def analysis_computation_create_scratch(
        ctx,
        analysis_config_id,
        start_date,
        end_date,
        aoi_file,
        date_format,
        frequency,
        dry_run,
        verbose,
):

    wf.check_environment_complete(raise_on_failure=True)
    click.echo("Using the following information:")
    click.echo(f"--> analysis_config_id:       {analysis_config_id}")
    click.echo(f"--> start_date:               {start_date}")
    click.echo(f"--> end_date:                 {end_date}")
    click.echo(f"--> aoi_file:                 {aoi_file}")
    click.echo(f"--> date_format:              {date_format}")
    click.echo(f"--> frequency:                {frequency}")

    if not dry_run:

        computation_id = asyncio.run(
            wf.create_analysis_computation_aoi_toi(
                analysis_config_id,
                start_date,
                end_date,
                aoi_file,
                date_format=date_format,
                frequency=frequency,
            )
        )

        click.echo(f"computation_id: {computation_id}")


@computation.command('run')
@click.pass_context
@click.option('-iP', '--analysis_computation_id', type=str)
@click.option('--dry_run', is_flag=True)
def analysis_computation_run(ctx, analysis_computation_id, dry_run):

    wf.check_environment_complete(raise_on_failure=True)
    if not dry_run:
        asyncio.run(wf.run_analysis_computations([analysis_computation_id]))


@computation.command('download')
@click.pass_context
@click.option('-iP', '--analysis_computation_id', type=str)
@click.option('-a', '--source_aoi_version', type=str)
@click.option('-os', '--min_observation_start_ts', type=str)
@click.option('-oe', '--max_observation_start_ts', type=str)
@click.option('-d', '--download_dir', type=str)
@click.option('-c', '--clobber', is_flag=True, help="Overwrite existing output")
def analysis_computation_download(
    ctx,
    analysis_computation_id,
    source_aoi_version=None,
    min_observation_start_ts=None,
    max_observation_start_ts=None,
    download_dir=None,
    clobber=False,
):

    wf.check_environment_complete(raise_on_failure=True)

    if os.path.exists(download_dir):
        if not clobber:
            click.secho(f"Directory: {download_dir} exists. Run with '-c' to overwrite.  Exiting.", fg='red')
            raise RuntimeError("Output file already exists.")
        else:
            click.secho(f"Directory: {download_dir} exists.  I'm removing it.", fg='yellow')
            os.rmdir(download_dir)

    asyncio.run(
        wf.download_algorithm_computation_results(
            analysis_computation_ids=[analysis_computation_id], source_aoi_version=source_aoi_version,
            min_observation_start_ts=min_observation_start_ts, max_observation_start_ts=max_observation_start_ts,
            download_dir=download_dir
        )
    )

    if not os.path.exists(download_dir):
        click.secho("NOT written", fg='red')
    else:
        click.secho("written.", fg='green')



import os
import warnings
from dataclasses import dataclass

from elements_api import ElementsAsyncClient

from elements.sdk.api.algorithm import APIAlgorithm, APIAlgorithmVersion, APIAlgorithmConfig, APIAlgorithmComputation
from elements.sdk.api.analysis import APIAnalysis, APIAnalysisVersion, APIAnalysisConfig, APIAnalysisComputation
from elements.sdk.api.aoi import APIAOI, APIAOITransaction, APIAOIVersion, APIAOICollection
from elements.sdk.api.credit import APICredit
from elements.sdk.api.data import APIDataSource, APIDataType
from elements.sdk.api.permission import APIPermission
from elements.sdk.api.result import APIResult
from elements.sdk.api.toi import APIToi
from elements.sdk.api.user import APIUser
from elements.sdk.api.visualization import APIVisualization
from elements.sdk.api.order import APIOrder
from elements.sdk.api.tasking_order import APITaskingOrder
from elements.sdk.api.filter import APIFilter, APIProjectFilter


@dataclass
class ElementsSDK:
    """
    https://docs.terrascope.orbitalinsight.com/docs
    """
    algorithm: APIAlgorithm
    algorithm_version: APIAlgorithmVersion
    algorithm_config: APIAlgorithmConfig
    algorithm_computation: APIAlgorithmComputation

    analysis: APIAnalysis
    analysis_version: APIAnalysisVersion
    analysis_config: APIAnalysisConfig
    analysis_computation: APIAnalysisComputation

    aoi: APIAOI
    aoi_transaction: APIAOITransaction
    aoi_version: APIAOIVersion
    aoi_collection: APIAOICollection

    toi: APIToi

    credit: APICredit

    data_source: APIDataSource
    data_type: APIDataType

    permission: APIPermission

    result: APIResult

    visualization: APIVisualization

    user: APIUser

    order: APIOrder

    tasking_order: APITaskingOrder

    filter: APIFilter

    project_filter: APIProjectFilter

    client: ElementsAsyncClient

    def __init__(self, client: ElementsAsyncClient = None, timeout: int = None):
        if timeout is None:
            timeout = int(os.getenv('ELEMENTS_TIMEOUT', default='60'))
        if client is None:
            client = self.create_client()

        # Set Up Algo APIs
        self.algorithm = APIAlgorithm(client=client, timeout=timeout)
        self.algorithm_version = APIAlgorithmVersion(client=client, timeout=timeout)
        self.algorithm_config = APIAlgorithmConfig(client=client, timeout=timeout)
        self.algorithm_computation = APIAlgorithmComputation(client=client, timeout=timeout)

        # Set Up Analysis APIs
        self.analysis = APIAnalysis(client=client, timeout=timeout)
        self.analysis_version = APIAnalysisVersion(client=client, timeout=timeout)
        self.analysis_config = APIAnalysisConfig(client=client, timeout=timeout)
        self.analysis_computation = APIAnalysisComputation(client=client, timeout=timeout)

        # Set Up AOI APIs
        self.aoi = APIAOI(client=client, timeout=timeout)
        self.aoi_transaction = APIAOITransaction(client=client, timeout=timeout)
        self.aoi_version = APIAOIVersion(client=client, timeout=timeout)
        self.aoi_collection = APIAOICollection(client=client, timeout=timeout)

        # Set Up AOI APIs
        self.toi = APIToi(client=client, timeout=timeout)

        # Set Up APICredit APIs
        self.credit = APICredit(client=client, timeout=timeout)

        # Set Up Data APIs
        self.data_type = APIDataType(client=client, timeout=timeout)
        self.data_source = APIDataSource(client=client, timeout=timeout)

        # Set Up Permission APIs
        self.permission = APIPermission(client=client, timeout=timeout)

        # Set Up APIResult APIs
        self.result = APIResult(client=client, timeout=timeout)

        # Set Up Visualization APIs
        self.visualization = APIVisualization(client=client, timeout=timeout)

        # Set Up User APIs
        self.user = APIUser(client=client, timeout=timeout)

        # set up order APIs
        self.order = APIOrder(client=client, timeout=timeout)

        # Set Up Tasking Order APIs
        self.tasking_order = APITaskingOrder(client=client, timeout=timeout)

        # Set Up Filter APIs
        self.filter = APIFilter(client=client, timeout=timeout)
        self.project_filter = APIProjectFilter(client=client, timeout=timeout)

    @staticmethod
    def create_client(elements_host=None, elements_api_token=None):
        # ELEMENTS_HOST
        if not elements_host:
            elements_host = os.getenv('ELEMENTS_HOST')
            if not elements_host:
                raise ValueError("ELEMENTS_HOST environment variable is required")

        # ELEMENTS_PORT
        elements_port_str = os.getenv('ELEMENTS_PORT', '443')
        try:
            elements_port = int(elements_port_str)
        except ValueError:
            raise ValueError(f"ELEMENTS_PORT must be an integer, got '{elements_port_str}'")

        # ELEMENTS_TOKEN
        if not elements_api_token:
            elements_api_token = os.getenv('ELEMENTS_TOKEN')
            if not elements_api_token:
                raise ValueError("ELEMENTS_TOKEN environment variable is required")

        # ELEMENTS_SECURE
        secure_string = os.getenv('ELEMENTS_SECURE', 'True')
        secure = secure_string.lower() in ['true', 'yes', '1']

        # Create client
        client = ElementsAsyncClient(
            elements_host,
            elements_port,
            api_token=elements_api_token,
            secure=secure
        )

        return client



#!/usr/bin/env python

import click
import sys
import asyncio
import pandas as pd
import elements.cli.lib.workflow as wf
import elements.cli.lib.utils as tsu
from elements.cli.lib.aliased_group import AliasedGroup


@click.command(cls=AliasedGroup, help="Analysis 'config' command group")
@click.pass_context
def config(ctx):
    pass


@config.command('get')
@click.pass_context
@click.option('-iC', '--analysis_config_id', type=str)
@click.option('-iV', '--analysis_version_id', type=str)
@click.option('-C', '--only_columns', multiple=True)
@click.option('-c', '--append_columns', multiple=True)
@click.option('-T', '--transpose', is_flag=True)
def analysis_config_get(
        ctx,
        analysis_config_id,
        analysis_version_id,
        only_columns,
        append_columns,
        transpose,
):

    ana_configs = asyncio.run(wf.get_analysis_configs(analysis_version_id=analysis_version_id,
                                                      analysis_config_id=analysis_config_id))

    columns = ['name', 'id', 'description', 'created_on']
    if only_columns:
        columns = list(only_columns)
    if append_columns:
        columns = columns + list(append_columns)

    data = []
    for ana_config in ana_configs:
        pbdict = tsu.protobuf_to_dict(ana_config)
        if 'analysis_configs' in pbdict:
            pbdict = pbdict['analysis_configs'][0]
        data.append(pbdict)
    df = pd.DataFrame(data)
    if 'all' not in columns:
        df = df[tsu.match_columns(df.columns, columns)]
    df = df.rename(columns={'id': 'analysis_config_id'})
    if transpose:
        df = df.T

    tsu.set_pandas_display()
    click.secho(f"analysis_version_id: {analysis_version_id}", fg='cyan')
    click.echo(df)


@config.command('list')
@click.pass_context
@click.option('-nC', '--analysis_config_name', type=str)
@click.option('-C', '--only_columns', multiple=True)
@click.option('-c', '--append_columns', multiple=True)
@click.option('-T', '--transpose', is_flag=True)
def analysis_config_list(
        ctx,
        analysis_config_name,
        only_columns,
        append_columns,
        transpose,
):

    ana_configs = asyncio.run(wf.list_analysis_configs(analysis_config_name=analysis_config_name))

    columns = ['name', 'id', 'created_on']
    if only_columns:
        columns = list(only_columns)
    if append_columns:
        columns = columns + list(append_columns)

    data = []
    for ana_config in ana_configs:
        pbdict = tsu.protobuf_to_dict(ana_config)
        if 'analysis_configs' in pbdict:
            pbdict = pbdict['analysis_configs'][0]
        data.append(pbdict)
    df = pd.DataFrame(data)
    if 'all' not in columns:
        df = df[tsu.match_columns(df.columns, columns)]
    df = df.rename(columns={'id': 'analysis_config_id'})
    if transpose:
        df = df.T

    tsu.set_pandas_display()

    click.echo(df)


@config.command('create')
@click.pass_context
@click.option('-iV', '--analysis_version_id', type=str)
@click.option('-ic', '--algorithm_config_id', type=str)
@click.option('-nC', '--analysis_config_name', type=str, help="This is the analysis name shown in the UI")
@click.option('-dC', '--analysis_config_desc', type=str, help="This is the TS description shownin the UI")
@click.option('--dry_run', is_flag=True)
def analysis_config_create(
        ctx,
        analysis_version_id,
        algorithm_config_id,
        analysis_config_name,
        analysis_config_desc,
        dry_run,
):

    wf.check_environment_complete(raise_on_failure=True)

    click.echo("Using the following information:")
    click.echo(f"--> analysis_version_id:      {analysis_version_id}")
    click.echo(f"--> algorithm_config_id:      {algorithm_config_id}")
    click.echo(f"--> analysis_config_name:     {analysis_config_name}")
    click.echo(f"--> analysis_config_desc:     {analysis_config_desc}")

    if not dry_run:
        analysis_config_id = asyncio.run(
            wf.create_analysis_config(
                analysis_version_id,
                algorithm_config_id,
                analysis_config_name,
                analysis_config_desc,
            )
        )
        tsu.echo_highlight_suffix("You're new analysis_config_id is: ", analysis_config_id, 'green')


@config.command('deactivate')
@click.pass_context
@click.option('-iC', '--analysis_config_ids', type=str, multiple=True, required=False,
              help="UUIDs of the analysis_configs to deactivate")
@click.option('-nC', '--analysis_config_name', type=str, required=False,
              help="Name of the analysis_config to deactivate (will match all)")
def analysis_config_deactivate(
        ctx,
        analysis_config_ids,
        analysis_config_name,
):

    if analysis_config_name:
        ana_configs = asyncio.run(wf.list_analysis_configs(analysis_config_name=analysis_config_name))

        data = []
        for ana_config in ana_configs:
            data.append([ana_config.id, ana_config.name])
        df = pd.DataFrame(data=data, columns=['analysis_config_id', 'analysis_config_name'])
        click.secho("I found these analysis_configs:", fg='red')
        click.echo(df)
        stdin = input("  DELETE ALL ?  [Y/N]: ")

        if stdin in ('Y', 'y'):
            click.secho("OK.  Deactivating analysis_configs", fg='red')
        else:
            click.secho("OK.  Exiting.", fg='red')
            sys.exit(0)

        analysis_config_ids += tuple(df['analysis_config_id'].values)

    response = asyncio.run(wf.deactivate_analysis_configs(analysis_config_ids=analysis_config_ids))

    click.echo("Deactivation response code: ", nl=False)
    if response:
        click.secho(f" {response} (FAILURE)", fg='red')
    else:
        click.secho(f" {response} (SUCCESS)", fg='green')



import logging
from datetime import datetime
from enum import Enum
from typing import List

from google.protobuf.timestamp_pb2 import Timestamp
from elements_api.models.toi_pb2 import TOI, Recurrence, Cadence


class Frequency(Enum):
    UNKNOWN_FREQUENCY = 0
    MINUTELY = 1
    HOURLY = 2
    DAILY = 3
    WEEKLY = 4
    MONTHLY = 5
    YEARLY = 6


class TOIBuilder:
    def __init__(self):
        self.__recurrences = []
        self.__toi = None
        self.__exclusion_dates = None

    def build_toi(self, start: datetime, finish: datetime, description: str = None):
        """
        The TOI service is responsible for determining what data a Computation should process.
        TOIs define the time period from which input data should be taken for a Computation or Analysis.
        The times specified in TOIs are not tied to time zones or UTC time - they are “floating” times.

        **Algorithm Impacts:**
        If an Algorithm has a *grouping* strategy of TOI defined in the algorithm manifest, then DEM will use the
        TOI configuration to ensure data delivered to an algorithm job meets the time interval unit specifications
        defined here.

        If an Algorithm does not have a *grouping* strategy of TOI defined in the algorithm manifest, then DEM will not
        consider time as a relevant data chunk and will deliver data to an Algorithm Job however it sees fit. The
        Algorithm must be designed in a way where the time groups DO NOT MATTER to the final results delivered.

        :param start: local start time (datetime)
        :param finish: local finish time (datetime)
        :param description: a description of the TOI
        :return:
        """
        start_ts = Timestamp()
        start_ts.FromDatetime(start)
        finish_ts = Timestamp()
        finish_ts.FromDatetime(finish)
        toi = TOI(
            start_local=start_ts,
            finish_local=finish_ts
        )
        if description is not None:
            toi.MergeFrom(TOI(description=description))
        self.__toi = toi

    def build_recurrence(self, rrule: str, cadence: Cadence = None):
        """
        Recurrences provide greater control over what data should be processed by a Computation. They replace the
        concept of dwell start and dwell end times in Platform 1.0.

        Recurrences include a rule and a duration. They leverage the power of iCalendar to define complex rules such as:

            - Monday-Friday for three months
            - Monday-Friday from 9am to 5pm for a year
            - Monday-Friday from 9am to 5pm for a year except holidays
            - Tuesday and Thursday every other week
            - Monday-Friday 8am to 6pm, Saturday-Sunday 9am to 5pm

        The rule maps to the iCalendar Recurrence Rule property. To learn more about recurrence rules, the rrule.js site
        provides an excellent online rule generator.

        online-rule-tool: https://jakubroztocil.github.io/rrule/

        :param rrule: Use RRuleBuilder.build_rule to get a valid rule string.
        :param cadence: [Optional] Cadence
        :return: None
        """
        if cadence is not None:
            self.__recurrences.append(Recurrence(rule=rrule, cadence=cadence))
        else:
            self.__recurrences.append(Recurrence(rule=rrule))

    def build_exclusion_dates(self, exclusion_dates: List[datetime]):
        """
        A list of dates
        :param exclusion_dates:
        :return:
        """
        for exclusion_date in exclusion_dates:
            self.__exclusion_dates.append(Timestamp().FromDateTime(exclusion_date))

    def get(self):
        assert self.__recurrences is not None
        toi = TOI()
        toi.MergeFrom(self.__toi)
        toi.MergeFrom(TOI(recurrences=self.__recurrences))
        if self.__exclusion_dates is not None:
            toi.MergeFrom(self.__exclusion_dates)
        return toi


class TOIRuleBuilder:
    @staticmethod
    def build_rule(**kwargs):
        """
        This is a basic class that follows the https://icalendar.org/RFC-Specifications/iCalendar-RFC-5545/
        specification. use this to construct rules for the given TOIBuilder

        UNTIL - If present, will override the TOI finish

         recur           = recur-rule-part *( ";" recur-rule-part )
                 ;
                 ; The rule parts are not ordered in any
                 ; particular sequence.
                 ;
                 ; The FREQ rule part is REQUIRED,
                 ; but MUST NOT occur more than once.
                 ;
                 ; The UNTIL or COUNT rule parts are OPTIONAL,
                 ; but they MUST NOT occur in the same 'recur'.
                 ;
                 ; The other rule parts are OPTIONAL,
                 ; but MUST NOT occur more than once.

        :param kwargs:
            - frequency: Frequency, the rate of occurrences,
            - interval: int, the step for the frequency. Frequency.Daily w/ interval=2 is every other day,
            - until: str, datetime format %Y%m%dT%H%M%SZ the future date to end by,
            - count: int, the number of occurrences,

        :return: rule: str - the RRule for a recurrence
        """
        valid_args = ['frequency', 'interval', 'until', 'count']
        assert len(kwargs.keys()) <= 3
        assert valid_args[0] and valid_args[1] in kwargs.keys()
        # Assume until and count are mutually exclusive to a rule
        assert valid_args[2] or valid_args[3] not in kwargs.keys()

        for key in kwargs.keys():
            if key not in valid_args:
                assert key in kwargs.keys()

        rule = "FREQ={};INTERVAL={}".format(Frequency(kwargs['frequency']).name, kwargs['interval'])
        if 'until' in kwargs.keys():
            logging.info("Rule contains [UNTIL] option - OVERRIDING TOI finish time")
            rule = rule + ";UNTIL={}".format(kwargs['until'])

        if 'count' in kwargs.keys():
            rule = rule + ";COUNT={}".format(kwargs['count'])

        return rule


class TOICadenceBuilder:
    @staticmethod
    def build_cadence(frequency: Frequency, value: int):
        """

        Cadences consist of a frequency and an interval. The frequency takes one of following values:
            - Minutely
            - Hourly
            - Daily
            - Weekly
            - Monthly
            - Yearly

        The interval is the value to apply to the frequency. Example group cadences would include:
            - 1 hour
            - 12 hours
            - 5 day
            - 2 weeks
            - 3 months
            - 1 year

        :param frequency:
        :param value:
        :return:
        """
        return Cadence(frequency=frequency, value=value)



#!/usr/bin/env python

import click
import asyncio
import pandas as pd
import elements.cli.lib.workflow as wf
import elements.cli.lib.utils as tsu
from elements.cli.lib.aliased_group import AliasedGroup


@click.command(cls=AliasedGroup, help="'toi' command group")
@click.pass_context
def toi(ctx):
    pass


@toi.command('get')
@click.pass_context
@click.option('-i', '--toi_ids', multiple=True, type=str)
@click.option('-v', '--verbose', is_flag=True)
def toi_get(ctx, toi_ids, verbose):

    tois = asyncio.run(wf.get_tois(toi_ids))
    data = []
    for toi in tois:
        data.append(tsu.protobuf_to_dict(toi))
    df = pd.DataFrame(data)
    columns = ['id', 'start_local', 'finish_local']
    df = df[columns]

    if verbose:
        click.echo(tois)
    click.echo(df)


@toi.command('create')
@click.pass_context
@click.option('-s', '--start_date', type=str)
@click.option('-e', '--end_date', type=str)
@click.option('-d', '--date_format', type=str, default='%Y-%m-%d')
@click.option('-f', '--frequency', type=click.Choice([2, 3]),
              help="HOURLY=2, DAILY=3", default=3)
def toi_create(ctx, start_date, end_date, date_format, frequency):

    if not start_date or not end_date:
        raise RuntimeError("Must provide start_date or end-date to 'create' a TOI")

    toi_id = asyncio.run(
        wf.create_toi(start_date, end_date, date_format=date_format, frequency=frequency)
    )
    tsu.echo_highlight_suffix("Your new toi_id is: ", toi_id, 'green')


@toi.command('delete')
@click.pass_context
@click.option('-i', '--toi_ids', multiple=True)
def toi_delete(ctx, toi_ids):

    asyncio.run(wf.delete_tois(toi_ids))



from datetime import datetime
from typing import List

from google.protobuf.json_format import MessageToDict
from google.protobuf.timestamp_pb2 import Timestamp
from elements_api import ElementsAsyncClient
from elements_api.models.algorithm_computation_pb2 import AlgorithmComputationGetRequest, \
    AlgorithmComputationCreateRequest, AlgorithmComputationRunRequest, AlgorithmComputation
from elements_api.models.algorithm_config_pb2 import AlgorithmConfig as AlgorithmConfig, \
    AlgorithmConfigGetRequest, AlgorithmConfigDeleteRequest, AlgorithmConfigCreateRequest, AlgorithmConfigUpdateRequest, \
    AlgorithmConfigDeprecateRequest, AlgorithmConfigDeactivateRequest, AlgorithmConfigListRequest
from elements_api.models.algorithm_pb2 import Algorithm, AlgorithmGetRequest, \
    AlgorithmUpdateRequest, AlgorithmCreateRequest, AlgorithmListRequest
from elements_api.models.algorithm_version_pb2 import AlgorithmVersion as AlgorithmVersion, \
    AlgorithmVersionGetRequest, \
    AlgorithmVersionCreateRequest, AlgorithmVersionListRequest, AlgorithmVersionDeprecateRequest, \
    AlgorithmVersionDeactivateRequest
from elements_api.models.common_models_pb2 import Pagination

from elements.sdk.builder.algorithm import AlgorithmManifest, AlgorithmConfiguration
from elements.sdk.tools.sdk_support import SDKSupport


class APIAlgorithm:

    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def create(self, name: str, display_name: str, author: str) -> Algorithm:
        """
        Description:
        Create a top level algorithm object. This object is lightweight and is used to connect all versions of an
        algorithm together. Once an algo has been created, an algo_version must be created using a manifest.
        Next an algo_config may be created to specify a particular configuration of an algorithm version, which can
        be used in an algo_computation.

        :return Algorithm
        """
        request = AlgorithmCreateRequest(
            name=name,
            display_name=display_name,
            author=author
        )
        response = await self.__client.api.algorithm.create(request, timeout=self.__timeout)
        return response.algorithm

    async def update(self, algorithms: List):
        """
        Update the mutable fields of a top level algorithm. This should be done rarely and with caution as it will
        affect all algorithm_versions, and algorithm_configs associated with this algorithm.

        :param algorithms: List[elements_api.models.algorithm_pb2.Algorithm]
        :return: EmptyResponse
        """

        request = AlgorithmUpdateRequest(
            algorithms=algorithms
        )
        await self.__client.api.algorithm.update(request, timeout=self.__timeout)

    async def get(self, algorithm_ids: List, pagination: Pagination = None) -> List[Algorithm]:
        """
        Retrieve the metadata of a particular algorithm.

        :param pagination:
        :param algorithm_ids:
        :return: List[Algorithm]
        """
        fetch_all = False
        if pagination is None:
            pagination = Pagination(
                page_size=100
            )
            fetch_all = True
        request = AlgorithmGetRequest(
            ids=algorithm_ids,
            pagination=pagination
        )

        if fetch_all:
            sdk_support = SDKSupport()
            responses = await sdk_support.get_all_paginated_objects(request,
                                                                    api_function=self.__client.api.algorithm.get,
                                                                    timeout=self.__timeout)
        else:
            responses = [await self.__client.api.algorithm.get(request)]
        algorithms = []
        for response in responses:
            algorithms.extend(response.algorithms)
        return algorithms

    async def list(self, **kwargs) -> List[Algorithm]:
        """
        Retrieve the metadata for all algos that this user has access to and that match the provided search filters,
        if any.
        :param kwargs:
            -Optional Params
            - search_text: str
            - min_created_on: Datetime
            - max_created_on: Datetime
            - pagination: Pagination
        :return: List[Algorithm]
        """
        parameters = ['search_text', 'min_created_on', 'max_created_on', 'pagination', 'raw_response']
        for key in kwargs.keys():
            assert key in parameters

        message_fragments = []
        fetch_all = False
        if 'pagination' not in kwargs.keys():
            message_fragments.append((AlgorithmListRequest(pagination=Pagination(
                page_size=1000
            ))))
            fetch_all = True

        if 'raw_response' not in kwargs.keys():
            kwargs['raw_response'] = False

        for key in kwargs.keys():
            if key == 'search_text':
                message_fragments.append(AlgorithmListRequest(search_text=kwargs[key]))
            if key == 'min_created_on':
                assert isinstance(kwargs[key], datetime)
                min_created_on = Timestamp()
                min_created_on.FromDatetime(kwargs[key])
                message_fragments.append(AlgorithmListRequest(min_created_on=min_created_on))
            if key == 'max_created_on':
                assert isinstance(kwargs[key], datetime)
                max_created_on = Timestamp()
                max_created_on.FromDatetime(kwargs[key])
                message_fragments.append(AlgorithmListRequest(max_created_on=max_created_on))
            if key == 'pagination':
                message_fragments.append(AlgorithmListRequest(pagination=kwargs[key]))

        request = AlgorithmListRequest()
        for fragment in message_fragments:
            request.MergeFrom(fragment)

        if fetch_all:
            sdk_support = SDKSupport()
            responses = await sdk_support.get_all_paginated_objects(request,
                                                                    api_function=self.__client.api.algorithm.list,
                                                                    timeout=self.__timeout)
            if kwargs['raw_response']:
                return responses
            else:
                algorithms = []
                for response in responses:
                    algorithms.extend(response.algorithms)
                return algorithms
        else:
            response = await self.__client.api.algorithm.list(request, timeout=self.__timeout)
            if kwargs['raw_response']:
                return response
            else:
                return response.algorithms


class APIAlgorithmVersion:

    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def create(self, algorithm_id: str, algorithm_manifest: AlgorithmManifest) -> AlgorithmVersion:
        """
        Given a top level algorithm, create a new version. Once an algorithm version has been created, it is immutable. Any changes require a new
        version to be created. Any semantically correct version supplied as part of the algorithm_manifest will be
        considered value and assigned to this algorithm_version. New versions will be required to be semantically
        greater than this version.

        Parameters
        :param algorithm_id:
        :param algorithm_manifest:
        :return: AlgorithmVersion
        """
        request = AlgorithmVersionCreateRequest(
            algorithm_id=algorithm_id,
            manifest_struct=algorithm_manifest.get()
        )
        response = await self.__client.api.algorithm_version.create(request, timeout=self.__timeout)
        return response.algorithm_version

    async def get(self, algorithm_version_ids: List, pagination: Pagination = None) -> List[AlgorithmVersion]:
        """
        Description:

        Get the details for a specific algorithm_version. By default it does not include the algorithm_manifest.
        Setting include_manifest to true, will also include the manifest in the response for each requested
        algorithm_version.

        :param pagination:
        :param algorithm_version_ids: The algorithm version ids to use to look up the sem versions associated
               with those algos
        :return: List[AlgorithmVersion]
        """
        if pagination is None:
            pagination = Pagination(
                page_size=1000
            )
        request = AlgorithmVersionGetRequest(
            ids=algorithm_version_ids,
            pagination=pagination
        )
        sdk_support = SDKSupport()
        responses = await sdk_support.get_all_paginated_objects(request,
                                                                api_function=self.__client.api.algorithm_version.get,
                                                                timeout=self.__timeout)
        algorithm_versions = []
        for response in responses:
            algorithm_versions.extend(response.algorithm_versions)
        return algorithm_versions

    async def list(self, **kwargs) -> List[AlgorithmVersion]:
        """
        List all the algorithm versions that the user has access to.

        **Search text** will be applied to the entire manifest returning all algorithm_versions
        whose manifest contains the search text.

        **Tags** will be matched exactly with the
        tags on the manifest.

        If **include_all_versions** is set to true, all prior versions of an algorithm including deprecated
        algorithm versions will be returned, as well. If include_manifest is set to true, the corresponding manifest
        of each algorithm_version will also be returned.

        By **default**, this method only returns the semantically latest version of each algorithm that has not been
        deprecated, It does not include manifests in the response.

        :param kwargs
            - algorithm_id: str
            - search_text: str
            - tag: str
            - min_created_on: Datetime
            - max_created_on: Datetime
            - include_all_versions: bool

        :return: List[AlgorithmVersion]
        """

        message_fragments = []
        fetch_all = False
        if 'pagination' not in kwargs.keys():
            message_fragments.append((AlgorithmVersionListRequest(pagination=Pagination(
                page_size=1000
            ))))
            fetch_all = True

        if 'raw_response' not in kwargs.keys():
            kwargs['raw_response'] = False


        for key in kwargs.keys():
            if key == 'algorithm_id':
                message_fragments.append(AlgorithmVersionListRequest(algorithm_id=kwargs[key]))
            if key == 'search_text':
                message_fragments.append(AlgorithmVersionListRequest(search_text=kwargs[key]))
            if key == 'min_created_on':
                assert isinstance(kwargs[key], datetime)
                min_created_on = Timestamp()
                min_created_on.FromDatetime(kwargs[key])
                message_fragments.append(AlgorithmVersionListRequest(min_created_on=min_created_on))
            if key == 'max_created_on':
                assert isinstance(kwargs[key], datetime)
                max_created_on = Timestamp()
                max_created_on.FromDatetime(kwargs[key])
                message_fragments.append(AlgorithmVersionListRequest(max_created_on=max_created_on))
            if key == 'include_all_versions':
                message_fragments.append(AlgorithmVersionListRequest(include_all_versions=kwargs[key]))
            if key == 'tag':
                message_fragments.append(AlgorithmVersionListRequest(tag=kwargs[key]))
            if key == 'pagination':
                message_fragments.append(AlgorithmVersionListRequest(pagination=kwargs[key]))

        request = AlgorithmVersionListRequest()
        for fragment in message_fragments:
            request.MergeFrom(fragment)
        MessageToDict(request)

        if fetch_all:
            sdk_support = SDKSupport()
            responses = await sdk_support.get_all_paginated_objects(request,
                                                                    api_function=self.__client.api.algorithm_version.list,
                                                                    timeout=self.__timeout)
            if kwargs['raw_response']:
                return responses
            else:
                algorithm_versions = []
                for response in responses:
                    algorithm_versions.extend(response.algorithm_versions)
                return algorithm_versions
        else:
            response = await self.__client.api.algorithm_version.list(request, timeout=self.__timeout)
            if kwargs['raw_response']:
                return response
            else:
                return response.algorithm_versions

    async def deprecate(self, ids: List):
        """
        Deprecate the specific version of this algorithm. Deprecated algorithms can still be searched for, metadata can
        be retrieved, and can still be used to create new algorithm_configs which can be used as part of
        algorithm_computations, but they are no longer actively supported. All algorithm_configs created from this
        algorithm_version are also deprecated, but can still be used in new algorithm_computations.

        :param ids: List[str]
        :return: EmptyResponse
        """
        request = AlgorithmVersionDeprecateRequest(
            ids=ids
        )
        await self.__client.api.algorithm_version.deprecate(request, timeout=self.__timeout)

    async def deactivate(self, algorithm_version_ids: List):
        """
        Deactivate the specific version of this algorithm. Metadata for deactivated algorithms can still be retrieved
        directly, but they will no longer show up in search results from the list endpoint.Deactivated
        algorithm_versions cannot be used to create new algorithm_configs. All algorithm_configs created from this
        algorithm_version are also deactivated and can no longer be used to create new algorithm_computations.
        Existing computations that are already using algorithm_configs created from this algorithm_version will still
        continue to run.

        :param algorithm_version_ids:
        :return:
        """
        request = AlgorithmVersionDeactivateRequest(
            ids=algorithm_version_ids
        )
        await self.__client.api.algorithm_version.deactivate(request, timeout=self.__timeout)


class APIAlgorithmConfig:

    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def create(self,
                     algorithm_version_id: str,
                     name: str,
                     description: str,
                     algorithm_config: AlgorithmConfiguration) -> AlgorithmConfig:
        """
        Creates a configuration of the specified algorithm version, including concrete values for all of the algorithm
        parameters. This configuration is stored in the system and the algorithm_config is returned to the user.

        :param algorithm_config:
        :param algorithm_version_id:
        :param name:
        :param description:
        :return: AlgorithmConfig
        """
        request = AlgorithmConfigCreateRequest(
            algorithm_version_id=algorithm_version_id,
            name=name,
            description=description,
            params=algorithm_config.get()
        )
        response = await self.__client.api.algorithm_config.create(request, timeout=self.__timeout)
        return response.algorithm_config

    async def update(self, algorithm_config_id: str, algorithm_config: AlgorithmConfiguration):
        """
        Update details of an algorithm_config, so long as it has not been locked. Once the algorithm_config is used
        in a computation, it is permanently locked and the specific parameter values can no longer be updated.

        :param algorithm_config_id:
        :param algorithm_config:
        :return:
        """
        request = AlgorithmConfigUpdateRequest(
            id=algorithm_config_id,
            algorithm_config=algorithm_config.get()
        )
        await self.__client.api.algorithm_config.update(request, timeout=self.__timeout)

    async def get(self, ids: List, pagination: Pagination = None) -> List[AlgorithmConfig]:
        """
        Retrieve the metadata of a particular algorithm configuration.

        :param ids:
        :param pagination:
        :return: List[AlgorithmConfig]
        """
        if pagination is not None:
            pagination = Pagination(
                page_size=1000
            )
        request = AlgorithmConfigGetRequest(
            ids=ids,
            pagination=pagination
        )
        sdk_support = SDKSupport()
        responses = await sdk_support.get_all_paginated_objects(request,
                                                                api_function=self.__client.api.algorithm_config.get,
                                                                timeout=self.__timeout)
        algorithm_configs = []
        for response in responses:
            algorithm_configs.extend(response.algorithm_configs)
        return algorithm_configs

    async def list(self, **kwargs) -> List[AlgorithmConfig]:
        """
        Retrieve the metadata for all algorithm configurations that this user has access to and that match the provided
        search filters. By default, deactivated algorithm_configs are not returned. Setting include_deactivated to true
        will also return any algorithm_configs that had been deactivated.

        :param kwargs:
            * algorithm_id: str,
            * algorithm_version_id: str,
            * search_text: str,
            * min_created_on: google.protobuf.Timestamp,
            * max_created_on: google.protobuf.Timestamp,
            * include_deactivated: bool,
            * pagination: Pagination

        :return: List[AlgorithmConfig]
        """
        message_fragments = []
        fetch_all = False
        if 'pagination' not in kwargs.keys():
            message_fragments.append(AlgorithmConfigListRequest(
                pagination=Pagination(
                    page_size=1000
                )))
            fetch_all = True
        if 'raw_response' not in kwargs.keys():
            kwargs['raw_response'] = False

        if 'include_deactivated' not in kwargs.keys():
            message_fragments.append(AlgorithmConfigListRequest(
                include_deactivated=False
            ))

        for key in kwargs.keys():
            if key == 'search_text':
                message_fragments.append(AlgorithmConfigListRequest(
                    search_text=kwargs[key]
                ))
            if key == 'algorithm_id':
                message_fragments.append(AlgorithmConfigListRequest(
                    algorithm_id=kwargs[key]
                ))
            if key == 'algorithm_version_id':
                message_fragments.append(AlgorithmConfigListRequest(
                    algorithm_version_id=kwargs[key]
                ))
            if key == 'min_created_on':
                assert isinstance(kwargs[key], datetime)
                min_created_on = Timestamp()
                min_created_on.FromDatetime(kwargs[key])
                message_fragments.append(AlgorithmListRequest(creation_date_min=min_created_on))
            if key == 'max_created_on':
                assert isinstance(kwargs[key], datetime)
                max_created_on = Timestamp()
                max_created_on.FromDatetime(kwargs[key])
                message_fragments.append(AlgorithmListRequest(creation_date_max=max_created_on))
            if key == 'include_deactivated':
                message_fragments.append(AlgorithmConfigListRequest(
                    include_deactivated=kwargs[key]
                ))
            if key == 'pagination':
                message_fragments.append(AlgorithmConfigListRequest(
                    pagination=kwargs[key]
                ))
        request = AlgorithmConfigListRequest()
        for fragment in message_fragments:
            request.MergeFrom(fragment)

        if fetch_all:
            sdk_support = SDKSupport()
            responses = await sdk_support.get_all_paginated_objects(request,
                                                                    api_function=self.__client.api.algorithm_config.list,
                                                                    timeout=self.__timeout)
            if kwargs['raw_response']:
                algorithm_configs = []
                for response in responses:
                    algorithm_configs.extend(response.algorithm_configs)
                return algorithm_configs
            else:
                return responses
        else:
            response = await self.__client.api.algorithm_config.list(request, timeout=self.__timeout)
            if kwargs['raw_response']:
                return response
            else:
                return response.algorithm_configs

    async def deprecate(self, algorithm_config_ids: List):
        """
        Deprecate the specific configuration of an algorithm_version. Deprecated algorithm_configs can still be
        searched for, metadata can be retrieved, and can still be used in new algorithm_computations, but they are no
        longer actively supported.

        :param algorithm_config_ids:
        :return:
        """
        request = AlgorithmConfigDeprecateRequest(
            ids=algorithm_config_ids
        )
        await self.__client.api.algorithm_config.deprecate(request, timeout=self.__timeout)

    async def deactivate(self, algorithm_config_ids: List):
        """
        Deactivate the specific configuration of an algorithm_version. Deactivated algorithm_configs will no longer
        show up in searches, but metadata can still be retrieved for deactivated algorithm_configs via the get endpoint.
        Deactivated algorithm_configs cannot be used in any new algorithm_computations, but existing
        algorithm_computations will continue to function properly.

        :param algorithm_config_ids:
        :return:
        """
        request = AlgorithmConfigDeactivateRequest(
            ids=algorithm_config_ids
        )
        await self.__client.api.algorithm_config.deactivate(request, timeout=self.__timeout)

    async def delete(self, algorithm_config_ids: List):
        """
        Delete the specified algorithm_config, if and only if it has not been used to create an algorithm_computation.
        Once an algorithm_config has been used to create an algorithm_computation it can never be deleted.
        The user must have permission to access the specified algorithm.

        :param algorithm_config_ids:
        :return: EmptyResponse
        """
        request = AlgorithmConfigDeleteRequest(
            ids=algorithm_config_ids
        )
        await self.__client.api.algorithm_config.delete(request, timeout=self.__timeout)


class APIAlgorithmComputation:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def create(self, algorithm_config_id: str, aoi_collection_id: str, toi_id: str) -> AlgorithmComputation:
        """
        Create the runnable component in the system, by specifiying the algorithm to be run, the set of AOIs to be run
        on, and the TOI to specify the recurrence. The algorithm_computation_id is returned so that it can be run.
        Note that until an algorithm_computation has been run, its underlying .resources are NOT locked and
        therefore can be changed.
        :param algorithm_config_id:
        :param aoi_collection_id:
        :param toi_id:
        :return: algorithm_computation_id
        """
        request = AlgorithmComputationCreateRequest(
            algorithm_config_id=algorithm_config_id,
            aoi_collection_id=aoi_collection_id,
            toi_id=toi_id
        )
        response = await self.__client.api.algorithm_computation.create(request)
        return response.algorithm_computation

    async def run(self, algorithm_computation_ids: List) -> int:
        """
        The first time this endpoint is called, the system initiates the running of the provided
        algorithm_computation_id. This causes credits to be place in pending status and eventually consumed when
        the computation completes. Credits are consumed as each result is produced. Once an algorithm_computation
        has been run, it is considered locked and cannot be changed, as are its constituents.

        :param algorithm_computation_ids:
        :return
        """
        request = AlgorithmComputationRunRequest(
            ids=algorithm_computation_ids
        )
        await self.__client.api.algorithm_computation.run(request)

    async def get(self, computation_ids: list, pagination: Pagination = None) -> List:
        """
        Description:
        This wrapper will get the status of your computations.

        :param pagination:
        :param computation_ids: The list of computations you want to get status for
        :return: List[AlgorithmComputation]
        """

        request = AlgorithmComputationGetRequest(
            ids=computation_ids,
            pagination=pagination)
        sdk_support = SDKSupport()
        responses = await sdk_support.get_all_paginated_objects(request,
                                                                api_function=self.__client.api.algorithm_computation.get,
                                                                timeout=self.__timeout)
        algorithm_computations = []
        for response in responses:
            algorithm_computations.extend(response.algorithm_computations)
        return algorithm_computations

    async def update(self):
        pass

    async def list(self, **kwargs):
        # message_fragments = []
        # if 'pagination' not in kwargs.keys():
        #     message_fragments.append(AlgorithmComputationListRequest(
        #         pagination=Pagination(
        #             page_size=100
        #         )))
        # if 'include_deactivated' not in kwargs.keys():
        #     message_fragments.append(AlgorithmComputationListRequest(
        #         include_deactivated=False
        #     ))
        #
        # for key in kwargs.keys():
        #     if key == 'search_text':
        #         message_fragments.append(AlgorithmComputationListRequest(
        #             search_text=kwargs[key]
        #         ))
        #     if key == 'algorithm_id':
        #         message_fragments.append(AlgorithmComputationListRequest(
        #             algorithm_id=kwargs[key]
        #         ))
        #     if key == 'algorithm_version_id':
        #         message_fragments.append(AlgorithmComputationListRequest(
        #             algorithm_version_id=kwargs[key]
        #         ))
        #     if key == 'min_created_on':
        #         assert isinstance(kwargs[key], datetime)
        #         min_created_on = Timestamp()
        #         min_created_on.FromDatetime(kwargs[key])
        #         message_fragments.append(AlgorithmComputationListRequest(creation_date_min=min_created_on))
        #     if key == 'max_created_on':
        #         assert isinstance(kwargs[key], datetime)
        #         max_created_on = Timestamp()
        #         max_created_on.FromDatetime(kwargs[key])
        #         message_fragments.append(AlgorithmComputationListRequest(creation_date_max=max_created_on))
        #     if key == 'include_deactivated':
        #         message_fragments.append(AlgorithmComputationListRequest(
        #             include_deactivated=kwargs[key]
        #         ))
        #     if key == 'pagination':
        #         message_fragments.append(AlgorithmComputationListRequest(
        #             pagination=kwargs[key]
        #         ))
        # request = AlgorithmComputationListRequest()
        # for fragment in message_fragments:
        #     request.MergeFrom(fragment)
        #
        # sdk_support = SDKSupport()
        # responses = await sdk_support.get_all_paginated_objects(request,
        #                                                         api_function=self.__client.api.algorithm_computation.list,
        #                                                         timeout=self.__timeout)
        # algorithm_computations = []
        # for response in responses:
        #     algorithm_computations.extend(response.algorithm_computations)
        # return algorithm_computations
        pass

    async def delete(self):
        pass

    async def pause(self):
        pass

    async def resume(self):
        pass

    async def history(self):
        pass



import uuid

import pytest
from elements_api.models.permission_pb2 import Permission
from elements_api.models.resource_pb2 import Resource

from elements.sdk.builder.algorithm import AlgorithmManifest, AlgorithmConfiguration, DataType, InterfaceType
from elements.sdk.builder.analysis import AnalysisManifest
from elements.sdk.elements_sdk import ElementsSDK


class TestPermission:

    def __init__(self):
        self.generate_resources()

    async def generate_resources(self):
        sdk = ElementsSDK()

        # Algorithm
        algo_hash = str(uuid.uuid4()).split("-")[0]
        algorithm = await sdk.algorithm.create(name="test-algo-permission-{}".format(algo_hash),
                                               author="test@orbitalinsight.com",
                                               display_name="Permission Test")
        algorithm_id = algorithm.id
        self.algorithm_id = algorithm_id
        print('algo id:', algorithm_id)

        # Algorithm Version (not shareable, to feed to config)
        manifest = AlgorithmManifest()
        manifest.metadata_required(description="test-manifest", version="0.0.1")
        manifest.interface_required(interface_type=InterfaceType.NO_OP_WORKER.value)
        manifest.inputs_add_data_type(DataType.DEVICE_VISITS.value)
        manifest.outputs_add_data_type(data_type_name=DataType.DEVICE_TRACKS.value,
                                       observation_value_columns=["device_tracks"])
        manifest.container_parameters_required(image="", command=[])
        algorithm_version = await sdk.algorithm_version.create(algorithm_id=algorithm_id, algorithm_manifest=manifest)
        print('algo version id:', algorithm_version.id)

        # Algorithm Config
        configuration = AlgorithmConfiguration()
        configuration.add_data_source(data_type=DataType.DEVICE_VISITS.value)
        algo_config = await sdk.algorithm_config.create(
            algorithm_version_id=algorithm_version.id,
            name="Permission Test Config",
            description="Permission Test Config",
            algorithm_config=configuration)
        algo_config_id = algo_config.id
        self.algo_config_id = algo_config_id
        print('algo config id:', algo_config_id)

        # Analysis
        analysis_hash = str(uuid.uuid4()).split("-")[0]
        analysis = await sdk.analysis.create(name="test-algo-permission-{}".format(analysis_hash),
                                             author="test@orbitalinsight.com")
        analysis_id = analysis.id
        self.analysis_id = analysis_id
        print('analysis id:', analysis_id)

        # Analysis Version
        analysis_manifest = AnalysisManifest()
        analysis_manifest.metadata(description="Permission Test Analysis Version",
                                   version="0.0.1", tags=['permission'])
        analysis_manifest.add_node(name="test-node", algorithm_version_id=algorithm_version.id)
        analysis_version = await sdk.analysis_version.create(analysis_id=analysis_id,
                                                             analysis_manifest=analysis_manifest)
        analysis_version_id = analysis_version.id
        print('analysis version id:', analysis_version_id)

        # # Analysis Config - breaking for some reason
        # confg = AnalysisConfiguration(analysis_version_id=analysis_version_id)
        # config.add_config_node(name="test-node", algorithm_config_id=algo_config_id)
        # analysis_config = await sdk.analysis_config.create(
        #     analysis_version_id=analysis_version_id,
        #     algorithm_config_nodes=config.get(),
        #     name="Permission Test Analysis Config",
        #     description="Permission Test Analysis Config")
        # analysis_config_id = analysis_config.id
        # self.analysis_config_id = analysis_config_id
        # print('analysis config id:', analysis_config_id)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("resource, permission_type", [
        (Resource.Type.ALGORITHM, Permission.Type.READ),
        (Resource.Type.ALGORITHM, Permission.Type.WRITE),
        (Resource.Type.ALGORITHM_CONFIG, Permission.Type.READ),
        (Resource.Type.ALGORITHM_CONFIG, Permission.Type.WRITE),
        (Resource.Type.ANALYSIS, Permission.Type.READ),
        (Resource.Type.ANALYSIS, Permission.Type.WRITE),
        # (Resource.Type.ANALYSIS_CONFIG, "read"),
        # (Resource.Type.ANALYSIS_CONFIG, "write"),
    ]
                             )
    async def test_create(self, resource_type: Resource.Type, permission_type: Permission.Type):
        sdk = ElementsSDK()
        if resource_type == Resource.Type.ALGORITHM:
            resource_id = self.algorithm_id
        elif resource_type == Resource.Type.ALGORITHM_CONFIG:
            resource_id = self.algo_config_id
        elif resource_type == Resource.Type.ANALYSIS:
            resource_id = self.analysis_id
        elif resource_type == Resource.Type.ANALYSIS_CONFIG:
            pass
        elif resource_type == Resource.Type.ALGORITHM_COMPUTATION:
            pass
        elif resource_type == Resource.Type.ANALYSIS_COMPUTATION:
            pass
        elif resource_type == Resource.Type.RESULT:
            pass
        elif resource_type == Resource.Type.TOI:
            pass
        elif resource_type == Resource.Type.AOI_COLLECTION:
            pass

        response = sdk.permission.create(resource_ids=[resource_id], resource_type=resource_type,
                                         permission_type=permission_type, user_emails=["test_user1@oi.com"],
                                         public=False, public_confirm=False)

        assert response is not None
        assert 200 <= response.status_code < 300



import time
import uuid
import pytest
from datetime import datetime
from typing import List, Tuple

from elements_api.models.common_models_pb2 import Pagination
from elements.sdk.api.aoi import AOIInputBuilder
from elements.sdk.builder.toi import TOIRuleBuilder, TOIBuilder, Frequency
from elements.sdk.builder.algorithm import (
    AlgorithmManifest, InterfaceType, DataType, DataSource, AlgorithmConfiguration
)
from elements.sdk.builder.analysis import AnalysisManifest, AnalysisConfiguration
from elements.sdk.elements_sdk import ElementsSDK


async def algorithm_create():
    sdk = ElementsSDK()
    algo_name = "device-visits-sdk-integration-test-{}".format(uuid.uuid4())
    algo_display_name = "Device Visits SDK Integration"
    algo_author = "elements.sdk.integration@orbitalinsight.com"

    algorithm = await sdk.algorithm.create(name=algo_name,
                                           display_name=algo_display_name,
                                           author=algo_author)
    assert algorithm.id is not None
    assert algorithm.name == algo_name
    assert algorithm.display_name == algo_display_name
    assert algorithm.author == algo_author
    return algorithm


async def algorithm_version_create(algorithm_id: str):
    sdk = ElementsSDK()
    manifest = AlgorithmManifest()
    manifest.metadata_required(
        description="Testing algo manifest builder",
    )
    manifest.interface_required(interface_type=InterfaceType.FILESYSTEM_TASK_WORKER.value)
    manifest.inputs_add_data_type(data_type_name=DataType.PINGS.value,
                                  min_count=1,
                                  max_count=2)

    manifest.container_parameters_required(image="orbitalinsight/raw_foottraffic:84c76f7f",
                                           command=["python",
                                                    "/orbital/base/algorithms/"
                                                    "raw_foottraffic/src/py/raw_foottraffic/simple_foottraffic.py"])
    manifest.outputs_add_data_type(data_type_name="device_visits",
                                   observation_value_columns=["unique_device_count"])

    algorithm_version = await sdk.algorithm_version.create(algorithm_id=algorithm_id,
                                                           algorithm_manifest=manifest)
    assert algorithm_version.id is not None
    return algorithm_version


async def algorithm_config_create(algorithm_version_id: str):
    sdk = ElementsSDK()
    algorithm_config_build = AlgorithmConfiguration()
    algorithm_config_build.add_data_source(data_type=DataType.PINGS.value, data_source=DataSource.WEJO)
    algorithm_config = await sdk.algorithm_config.create(algorithm_version_id=algorithm_version_id,
                                                         algorithm_config=algorithm_config_build,
                                                         name="test_config-{}".format(uuid.uuid4()),
                                                         description="sdk test config")
    return algorithm_config


async def analysis_create(name, author):
    sdk = ElementsSDK()
    analysis = await sdk.analysis.create(name=name,
                                         author=author)
    assert analysis.id is not None
    assert analysis.name == name
    assert analysis.author == author
    return analysis


async def analysis_version_create(analysis_id: str, algorithm_version_ids: List[str],
                                  node_edges: List[Tuple[int, int]]):
    sdk = ElementsSDK()
    manifest = AnalysisManifest()
    description = "Test description for the greatest manifest in the world."
    manifest.metadata(description=description, tags=["sdk-test", "cap-sdk"])
    for algorithm_version_id in algorithm_version_ids:
        manifest.add_node(name=f"fake_name_{str(uuid.uuid4())}",
                          algorithm_version_id=algorithm_version_id)
    for source_node, dest_node in node_edges:
        manifest.add_node_edge(source_node, dest_node)

    analysis_version = await sdk.analysis_version.create(analysis_id=analysis_id, analysis_manifest=manifest)
    assert analysis_version.id is not None
    return analysis_version, manifest


async def analysis_config_create(analysis_version_id, algorithm_config_id, manifest):
    sdk = ElementsSDK()
    config = AnalysisConfiguration(analysis_version_id)
    config.add_config_node(name=manifest.get().algorithm_nodes[0].name,
                           algorithm_config_id=algorithm_config_id)
    analysis_config = await sdk.analysis_config.create(analysis_version_id=analysis_version_id,
                                                       name="analysis_config_test_create_{}".format(uuid.uuid4()),
                                                       description="Integration test for analysis_config.create",
                                                       algorithm_config_nodes=config.get())
    assert analysis_config.id is not None
    return analysis_config


async def analysis_computation_create(analysis_config_id: str, toi_id: str, aoi_collection_id: str):
    sdk = ElementsSDK()
    analysis_computation = await sdk.analysis_computation.create(
        analysis_config_id=analysis_config_id,
        toi_id=toi_id,
        aoi_collection_id=aoi_collection_id)
    assert analysis_computation.id is not None
    return analysis_computation


async def aoi_collection_create(name):
    sdk = ElementsSDK()
    aoi_collection = await sdk.aoi_collection.create(aoi_collection_name=name)
    assert aoi_collection.id is not None

    wkt = """POLYGON ((-121.891336 37.345116, -121.882978 37.322622, -121.865618 37.335404, -121.891336 37.345116))
            """
    aoi_builder = AOIInputBuilder()
    aoi = aoi_builder.build(
        geom_wkt=wkt,
        name="aoi-" + name,
        category="industry",
        category_type="LNG",
        tags=["industrial", "LNG", "test"])
    aoi_identifiers = await sdk.aoi.create(aoi_collection_id=aoi_collection.id, aoi_inputs=[aoi])
    assert aoi_identifiers is not None
    return aoi_collection


async def toi_create():
    sdk = ElementsSDK()
    datetime_format = '%Y-%m-%dT%H:%M:%SZ'
    toi_configuration = TOIBuilder()
    toi_configuration.build_toi(start=datetime.strptime("2019-01-05T01:00:00Z", datetime_format),
                                finish=datetime.strptime("2019-01-06T01:00:00Z", datetime_format))
    toi_configuration.build_recurrence(TOIRuleBuilder.build_rule(
        frequency=Frequency.DAILY,
        interval=1
    ))
    toi = await sdk.toi.create(toi_configuration.get())
    assert toi.id is not None
    return toi


class TestAnalysis:
    @pytest.mark.asyncio
    async def test_create(self):
        await analysis_create(name="integration-test-{}".format(datetime.now()), author="elements-sdk")

    @pytest.mark.asyncio
    async def test_update(self):
        pass

    @pytest.mark.asyncio
    @pytest.mark.parametrize("analysis_id_count", [
        1,
        10,
        25,
        45
    ])
    async def test_get(self, analysis_id_count: int):
        sdk = ElementsSDK()
        analysis_ids = []
        for i in range(analysis_id_count):
            analysis = await analysis_create(name="analysis_get_element_{}".format(uuid.uuid4()),
                                             author="elements-sdk-pytest")
            analysis_ids.append(analysis.id)
        assert len(analysis_ids) == analysis_id_count
        analyses = await sdk.analysis.get(ids=analysis_ids,
                                          pagination=Pagination(page_size=5))
        assert len(analyses) == analysis_id_count

    @pytest.mark.asyncio
    @pytest.mark.parametrize("analysis_count", [
        10,
        20
    ])
    async def test_list(self, analysis_count):
        sdk = ElementsSDK()
        start = datetime.utcnow()
        halfway = None
        search_text = uuid.uuid4()
        for i in range(analysis_count):
            await analysis_create(name="sdk-search-me-{}-{}".format(i, search_text),
                                  author="elements-sdk-list-integration")
            if i + 1 == analysis_count / 2:
                time.sleep(10)
                halfway = datetime.utcnow()

        results = await sdk.analysis.list(search_text=str(search_text))
        assert len(results) == analysis_count

        search_results = await sdk.analysis.list(min_created_on=start, max_created_on=halfway)
        assert len(search_results) == analysis_count / 2


class TestAnalysisVersion:
    @pytest.mark.asyncio
    async def test_create(self):
        algorithm = await algorithm_create()
        algorithm_version_ids = []
        for i in range(3):
            algorithm_version = await algorithm_version_create(algorithm.id)
            algorithm_version_ids.append(algorithm_version.id)
        analysis = await analysis_create(name="analysis-version-create-test-{}".format(uuid.uuid4()),
                                         author="elements-sdk")
        await analysis_version_create(analysis.id, algorithm_version_ids, [(0, 1), (1, 2)])

    @pytest.mark.asyncio
    @pytest.mark.parametrize("analysis_version_id_count", [
        1,
        10,
        25,
        45
    ])
    async def test_get(self, analysis_version_id_count):
        sdk = ElementsSDK()
        algorithm = await algorithm_create()
        algorithm_version_ids = []
        for i in range(3):
            algorithm_version = await algorithm_version_create(algorithm.id)
            algorithm_version_ids.append(algorithm_version.id)
        analysis = await analysis_create(name="analysis-version-get-test-{}".format(datetime.now()),
                                         author="elements-sdk")
        analysis_version_ids = []
        for i in range(analysis_version_id_count):
            analysis_version, _ = await analysis_version_create(analysis.id, algorithm_version_ids, [(0, 1), (1, 2)])
            analysis_version_ids.append(analysis_version.id)
        assert len(analysis_version_ids) == analysis_version_id_count

        # Algo details should fail since fake algo
        analysis_versions = await sdk.analysis_version.get(ids=analysis_version_ids,
                                                           include_manifest=True,
                                                           pagination=Pagination(page_size=5))

        assert len(analysis_versions) == analysis_version_id_count

    @pytest.mark.asyncio
    @pytest.mark.parametrize("analysis_version_id_count", [
        10,
        20
    ])
    async def test_list(self, analysis_version_id_count: int):
        sdk = ElementsSDK()
        algorithm = await algorithm_create()
        algorithm_version_ids = []
        for i in range(3):
            algorithm_version = await algorithm_version_create(algorithm.id)
            algorithm_version_ids.append(algorithm_version.id)
        analysis = await analysis_create(name="analysis-version-list-test-{}".format(datetime.now()),
                                         author="elements-sdk")

        min_created_on = datetime.utcnow()
        max_created_on = None
        analysis_version_ids = []
        for i in range(analysis_version_id_count):
            analysis_version, _ = await analysis_version_create(analysis.id, algorithm_version_ids, [(0, 1), (1, 2)])
            if i == (analysis_version_id_count / 2) - 1:
                time.sleep(10)
                max_created_on = datetime.utcnow()

            analysis_version_ids.append(analysis_version.id)

        analysis_versions = await sdk.analysis_version.list(analysis_id=analysis.id, include_all_versions=False)

        assert len(analysis_versions) == 1

        analysis_versions = await sdk.analysis_version.list(analysis_id=analysis.id,
                                                            include_all_versions=True)
        assert len(analysis_versions) == len(analysis_version_ids)
        for version in analysis_versions:
            assert version.analysis_id == analysis.id

        analysis_versions = await sdk.analysis_version.list(min_created_on=min_created_on,
                                                            max_created_on=max_created_on,
                                                            include_all_versions=True)
        assert len(analysis_versions) == analysis_version_id_count / 2

    @pytest.mark.asyncio
    async def test_deprecate(self):
        pass

    @pytest.mark.asyncio
    async def test_deactivate(self):
        pass


class TestAnalysisConfig:
    @pytest.mark.asyncio
    async def test_create(self):
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        analysis = await analysis_create(name="analysis-config-create-test-{}".format(datetime.now()),
                                         author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        algorithm_config = await algorithm_config_create(algorithm_version.id)
        await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)

    @pytest.mark.asyncio
    async def test_update(self):
        pass

    @pytest.mark.asyncio
    @pytest.mark.parametrize("config_count", [
        1,
        10,
        25,
        45
    ])
    async def test_get(self, config_count: int):
        sdk = ElementsSDK()
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        analysis = await analysis_create(name="analysis-config-get-test-{}".format(datetime.now()),
                                         author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        algorithm_config = await algorithm_config_create(algorithm_version.id)
        analysis_config_ids = []
        for i in range(config_count):
            analysis_config = await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)
            analysis_config_ids.append(analysis_config.id)

        analysis_configs = await sdk.analysis_config.get(ids=analysis_config_ids,
                                                         include_algorithm_details=False,
                                                         pagination=Pagination(page_size=5))

        assert len(analysis_configs) == config_count

    @pytest.mark.asyncio
    @pytest.mark.parametrize("config_count", [
        1,
        10,
        25
    ])
    async def test_list(self, config_count: int):
        sdk = ElementsSDK()
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        analysis = await analysis_create(name="analysis-config-list-test-{}".format(datetime.now()),
                                         author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        algorithm_config = await algorithm_config_create(algorithm_version.id)

        analysis_config_ids = []
        for i in range(config_count):
            analysis_config = await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)
            analysis_config_ids.append(analysis_config.id)

        analysis_configs = await sdk.analysis_config.list(analysis_version_id=analysis_version.id)
        assert len(analysis_configs) == config_count

    @pytest.mark.asyncio
    async def test_deprecate(self):
        pass

    @pytest.mark.asyncio
    @pytest.mark.parametrize("config_count", [
        1,
        10,
        25
    ])
    @pytest.mark.asyncio
    async def test_deactivate(self, config_count: int):
        sdk = ElementsSDK()
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        analysis = await analysis_create(name="analysis-config-list-test-{}".format(datetime.now()),
                                         author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        algorithm_config = await algorithm_config_create(algorithm_version.id)

        analysis_config_ids = []
        for i in range(config_count):
            analysis_config = await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)
            analysis_config_ids.append(analysis_config.id)

        # DEACTIVATE ANALYSIS CONFIG
        await sdk.analysis_config.deactivate(analysis_config_ids)

        # Check that List request doesn't return de-activated Analysis Configs unless specified
        analysis_configs = await sdk.analysis_config.list(analysis_version_id=analysis_version.id)
        assert len(analysis_configs) == 0
        analysis_configs = await sdk.analysis_config.list(analysis_version_id=analysis_version.id,
                                                          include_deactivated=True)
        assert len(analysis_configs) == config_count

        # Check that Analysis Configs can still be queried through the get request, but has deactivated flag set
        analysis_configs = await sdk.analysis_config.get(ids=analysis_config_ids, include_algorithm_details=False)
        assert all([analysis_config.is_deactivated for analysis_config in analysis_configs])

    @pytest.mark.asyncio
    async def test_delete(self):
        pass


class TestAnalysisComputation:
    @pytest.mark.asyncio
    async def test_create(self):
        # SETUP ALGORITHM
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        algorithm_config = await algorithm_config_create(algorithm_version.id)

        # SETUP ANALYSIS
        name = "analysis-computation-create-test-{}".format(datetime.now())
        analysis = await analysis_create(name=name, author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        analysis_config = await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)

        # SETUP AOI
        aoi_collection = await aoi_collection_create(name)

        # SETUP TOI
        toi = await toi_create()

        # SETUP ANALYSIS COMPUTATION
        await analysis_computation_create(analysis_config.id, toi.id, aoi_collection.id)

    @pytest.mark.asyncio
    async def test_run(self):
        sdk = ElementsSDK()
        # SETUP ALGORITHM
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        algorithm_config = await algorithm_config_create(algorithm_version.id)

        # SETUP ANALYSIS
        name = "analysis-computation-run-test-{}".format(datetime.now())
        analysis = await analysis_create(name=name, author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        analysis_config = await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)

        # SETUP AOI
        aoi_collection = await aoi_collection_create(name)

        # SETUP TOI
        toi = await toi_create()

        # SETUP ANALYSIS COMPUTATION
        analysis_computation = await analysis_computation_create(analysis_config.id, toi.id, aoi_collection.id)

        # RUN ANALYSIS COMPUTATION
        await sdk.analysis_computation.run([analysis_computation.id])

    @pytest.mark.asyncio
    async def test_get(self):
        sdk = ElementsSDK()
        # SETUP ALGORITHM
        algorithm = await algorithm_create()
        algorithm_version = await algorithm_version_create(algorithm.id)
        algorithm_config = await algorithm_config_create(algorithm_version.id)

        # SETUP ANALYSIS
        name = "analysis-computation-get-test-{}".format(datetime.now())
        analysis = await analysis_create(name=name, author="elements-sdk")
        analysis_version, manifest = await analysis_version_create(analysis.id, [algorithm_version.id], [])
        analysis_config = await analysis_config_create(analysis_version.id, algorithm_config.id, manifest)

        # SETUP AOI
        aoi_collection = await aoi_collection_create(name)

        # SETUP TOI
        toi = await toi_create()

        # SETUP ANALYSIS COMPUTATION
        analysis_computation = await analysis_computation_create(analysis_config.id, toi.id, aoi_collection.id)

        # GET ANALYSIS COMPUTATION
        analysis_computation = await sdk.analysis_computation.get([analysis_computation.id])
        assert analysis_computation is not None

    @pytest.mark.asyncio
    async def test_list(self):
        pass



import uuid

from elements.sdk.builder.analysis import AnalysisManifest


class TestAnalysisManifest:
    def test_manifest(self):
        manifest = AnalysisManifest()
        manifest_version = "0.0.1"
        description = "Test description for the greatest manifest in the world."
        version = "0.1.0"
        manifest.metadata(description=description,
                          version=version,
                          tags=["sdk-test", "cap-sdk"])
        manifest.add_node(name="fake_name_1",
                          algorithm_version_id=str(uuid.uuid4()))
        manifest.add_node(name="fake_name_child_2",
                          algorithm_version_id=str(uuid.uuid4()))
        manifest.add_node(name="fake_name_child_3",
                          algorithm_version_id=str(uuid.uuid4()))

        manifest.add_node_edge(0, 1)
        manifest.add_node_edge(1, 2)

        manifest_struct = manifest.get()
        assert manifest_struct.metadata.description == description
        assert manifest_struct.metadata.version == version
        assert len(manifest_struct.algorithm_nodes) == 3



import uuid
import pytest
import os.path
import logging
from datetime import datetime

from elements.sdk.builder.toi import TOIBuilder, TOIRuleBuilder, Frequency
from elements.sdk.builder.algorithm import (
    AlgorithmManifest, AlgorithmConfiguration, InterfaceType, DataType, DataSource
)
from elements.sdk.builder.analysis import AnalysisManifest, AnalysisConfiguration
from elements.sdk.elements_sdk import ElementsSDK


class TestResult:

    @pytest.mark.asyncio
    # @pytest.mark.parametrize()
    async def test_result_get_export(self):
        # SETUP ALGORITHM
        sdk = ElementsSDK()
        algorithm_name = "device-visits-sdk-integration-test-{}".format(uuid.uuid4())
        algorithm_display_name = "Device Visits SDK Integration"
        algorithm_author = "elements.sdk.integration@orbitalinsight.com"

        algorithm = await sdk.algorithm.create(name=algorithm_name,
                                               display_name=algorithm_display_name,
                                               author=algorithm_author)
        assert algorithm.id is not None
        assert algorithm.name == algorithm_name
        assert algorithm.display_name == algorithm_display_name
        assert algorithm.author == algorithm_author

        manifest = AlgorithmManifest()
        manifest.metadata_required(description="Testing algo manifest builder")
        manifest.interface_required(interface_type=InterfaceType.FILESYSTEM_TASK_WORKER.value)
        manifest.inputs_add_data_type(data_type_name=DataType.PINGS.value)

        manifest.container_parameters_required(image="orbitalinsight/device_visits:1d77d321",
                                               command=["python",
                                                        "/orbital/base/algorithms/device_visits/src/py/"
                                                        "device_visits/device_visits.py"])
        manifest.outputs_add_data_type(data_type_name="device_visits", observation_value_columns=["device_visits"])

        manifest.parameter_add(name="look_back_time", type="integer", default=3600,
                               description="how far to look back")
        manifest.parameter_add(name="look_forward_time", type="integer", default=3600,
                               description="how far to look back")

        manifest.container_parameters_resource_request(memory_gb=5, cpu_millicore=200)
        algorithm_version = await sdk.algorithm_version.create(algorithm_id=algorithm.id,
                                                               algorithm_manifest=manifest)

        assert algorithm_version.id is not None
        await sdk.credit.set_algorithm(algorithm_version_id=algorithm_version.id,
                                       algorithm_execution_price=1.0,
                                       algorithm_value_price=0.0)

        configuration = AlgorithmConfiguration()
        configuration.add_data_source(DataType.PINGS.value, DataSource.WEJO)

        algorithm_configuration = await sdk.algorithm_config.create(algorithm_version_id=algorithm_version.id,
                                                                    name="device_visit_sdk_test_config",
                                                                    description="A test configuration.",
                                                                    algorithm_config=configuration)
        # AOI Setup
        aoi_collection = await sdk.aoi_collection.create(aoi_collection_name=str(uuid.uuid4()))
        assert aoi_collection.id is not None
        await sdk.aoi.upload(aoi_collection_id=aoi_collection.id,
                             file_path="../resources/aois/geojson/us-amz-distro-centers.geojson")
        # TOI Setup
        datetime_format = '%Y-%m-%dT%H:%M:%SZ'
        toi_configuration = TOIBuilder()
        toi_configuration.build_toi(start=datetime.strptime("2019-01-05T01:00:00Z", datetime_format),
                                    finish=datetime.strptime("2019-01-06T01:00:00Z", datetime_format))
        toi_configuration.build_recurrence(TOIRuleBuilder.build_rule(
            frequency=Frequency.DAILY,
            interval=1
        ))
        toi = await sdk.toi.create(toi_configuration.get())
        assert toi.id is not None

        # Analysis Setup
        name = "analysis-config-create-test-{}".format(uuid.uuid4())
        author = "elements-sdk"
        analysis = await sdk.analysis.create(name=name,
                                             author=author)
        assert analysis.id is not None
        assert analysis.name == name
        assert analysis.author == author

        manifest = AnalysisManifest()
        description = "Test description for the greatest manifest in the world."
        version = "0.0.1"
        manifest.metadata(description=description,
                          version=version,
                          tags=["device-visits", "result-test"])
        manifest.add_node(name="device-visits",
                          algorithm_version_id=algorithm_version.id)
        analysis_version = await sdk.analysis_version.create(analysis_id=analysis.id,
                                                             analysis_manifest=manifest)

        config = AnalysisConfiguration(analysis_version.id)
        config.add_config_node(name=manifest.get().algorithm_nodes[0].name,
                               algorithm_config_id=algorithm_configuration.id)
        analysis_config = await sdk.analysis_config.create(analysis_version_id=analysis_version.id,
                                                           name="Results Analysis - Device Visits-{}".format(str(uuid.uuid4()).split("-")[0]),
                                                           description="Integration test for analysis_config.create",
                                                           algorithm_config_nodes=config.get())

        assert analysis_config.id is not None

        analysis_computation = await sdk.analysis_computation.create(analysis_config_id=analysis_config.id,
                                                                     toi_id=toi.id,
                                                                     aoi_collection_id=aoi_collection.id)

        logging.info(analysis_computation)
        assert analysis_computation.id is not None
        await sdk.analysis_computation.run(analysis_computation_ids=[analysis_computation.id])

        algorithm_computation_id = analysis_computation.computation_nodes[0].computation_id
        source_aoi_version = 10  # How to get this?
        await sdk.result.wait_and_download(algorithm_computation_ids=[algorithm_computation_id],
                                           output="../output/test-result-wejo-amzdst-{}.zip".format(
                                               algorithm_computation_id),
                                           source_aoi_version=source_aoi_version)

        assert os.path.isfile("../output/test-result-wejo-amzdst-{}.zip".format(algorithm_computation_id))

    @pytest.mark.asyncio
    async def test_result_export(self):
        sdk = ElementsSDK()
        # algorithm_computation_id for prod env
        algorithm_computation_id = '5ce2d9b4-170f-42ad-9c18-21398e45c72b'
        source_aoi_version = 36091042
        file_paths = await sdk.result.export(algorithm_computation_id, source_aoi_version)
        for file_path in file_paths:
            assert os.path.exists(file_path)



#!/usr/bin/env python

import click
import asyncio
import os
import elements.cli.lib.utils as tsu
from elements.cli.lib.aliased_group import AliasedGroup
import elements.cli.lib.workflow as wf


@click.group(cls=AliasedGroup, help="Algorithm 'computation' command group")
@click.pass_context
def computation(ctx):
    pass


@computation.command(name='get')
@click.pass_context
@click.option('-ip', '--algorithm_computation_id', type=str)
@click.option('-v', '--verbose', is_flag=True)
def algorithm_computation_get(ctx, algorithm_computation_id, verbose):

    wf.check_environment_complete(raise_on_failure=True)

    algorithm_computation_info = asyncio.run(
        wf.get_algorithm_computation_info([algorithm_computation_id])
    )[0]
    info = tsu.protobuf_to_dict(algorithm_computation_info)

    click.secho(algorithm_computation_id, fg='cyan')
    for k, v in info.items():
        click.echo(f"{k:20s} {v}")


@computation.command(name='create')
@click.pass_context
@click.option('-ic', '--algorithm_config_id', type=str)
@click.option('-iac', '--aoi_collection_id', type=str)
@click.option('-it', '--toi_id', type=str)
@click.option('-s', '--start_date', type=str)
@click.option('-e', '--end_date', type=str)
@click.option('-af', '--aoi_file', type=str)
@click.option('-d', '--date_format', type=str, default='%Y-%m-%d')
@click.option('-f', '--frequency', type=click.IntRange(min=2, max=3, clamp=False), default=3,
              help="HOURLY=2, DAILY=3")
@click.option('--dry_run', is_flag=True)
@click.option('-v', '--verbose', is_flag=True)
def algorithm_computation_create(
        ctx,
        algorithm_config_id,
        start_date,
        end_date,
        aoi_file,
        date_format,
        frequency,
        dry_run,
        verbose,
        aoi_collection_id=None,
        toi_id=None,
):

    wf.check_environment_complete(raise_on_failure=True)
    click.echo("Using the following information:")
    click.echo(f"--> algorithm_config_id:      {algorithm_config_id}")
    click.echo(f"--> start_date:               {start_date}")
    click.echo(f"--> end_date:                 {end_date}")
    click.echo(f"--> aoi_file:                 {aoi_file}")
    click.echo(f"--> date_format:              {date_format}")
    click.echo(f"--> frequency:                {frequency}")
    click.echo(f"--> aoi_collectin_id:         {aoi_collection_id}")
    click.echo(f"--> toi_id:                   {toi_id}")

    if not dry_run:

        if aoi_collection_id and toi_id:
            computation_id = asyncio.run(
                wf.create_algorithm_computation(
                    algorithm_config_id,
                    toi_id,
                    aoi_collection_id,
                )
            )
        else:
            computation_id = asyncio.run(
                wf.create_algorithm_computation_aoi_toi(
                    algorithm_config_id,
                    start_date,
                    end_date,
                    aoi_file,
                    date_format=date_format,
                    frequency=frequency,
                )
            )

        tsu.echo_highlight_suffix("computation_id: ", computation_id, 'green')


@computation.command(name='run')
@click.pass_context
@click.option('-ip', '--algorithm_computation_id', type=str)
@click.option('--dry_run', is_flag=True)
@click.option('-v', '--verbose', is_flag=True)
def algorithm_computation_run(
        ctx,
        algorithm_computation_id,
        dry_run,
        verbose,
):

    wf.check_environment_complete(raise_on_failure=True)
    click.echo(f"Running algorithm_computation_id={algorithm_computation_id}  (dry_run={dry_run})")
    if not dry_run:
        asyncio.run(wf.run_algorithm_computations([algorithm_computation_id]))


@computation.command(name='download')
@click.pass_context
@click.option('-ip', '--algorithm_computation_id', type=str)
@click.option('-a', '--source_aoi_version', type=str)
@click.option('-os', '--min_observation_start_ts', type=str)
@click.option('-oe', '--max_observation_start_ts', type=str)
@click.option('-d', '--download_dir', type=str)
@click.option('-c', '--clobber', is_flag=True, help="Overwrite existing output")
def algorithm_computation_download(
    ctx,
    algorithm_computation_id,
    source_aoi_version=None,
    min_observation_start_ts=None,
    max_observation_start_ts=None,
    download_dir=None,
    clobber=False,
):
    wf.check_environment_complete(raise_on_failure=True)

    if os.path.exists(download_dir):
        if not clobber:
            click.secho(f"Directory: {download_dir} exists. Run with '-c' to overwrite.  Exiting.", fg='red')
            raise RuntimeError("Output file already exists.")
        else:
            click.secho(f"Directory: {download_dir} exists.  I'm removing it.", fg='yellow')
            os.rmdir(download_dir)

    asyncio.run(
        wf.download_algorithm_computation_results(
            algorithm_computation_ids=[algorithm_computation_id], source_aoi_version=source_aoi_version,
            min_observation_start_ts=min_observation_start_ts, max_observation_start_ts=max_observation_start_ts,
            download_dir=download_dir
        )
    )

    if not os.path.exists(download_dir):
        click.secho("NOT written", fg='red')
    else:
        click.secho("written.", fg='green')



import pytest

from google.protobuf.json_format import MessageToDict
from elements.sdk.builder.algorithm import AlgorithmManifest, InterfaceType, DataType


class TestAlgorithmManifest:
    @pytest.mark.asyncio
    async def test_algorithm_manifest(self):
        manifest = AlgorithmManifest()
        manifest.metadata_required(description="Testing algo manifest builder")
        manifest.interface_required(interface_type=InterfaceType.FILESYSTEM_TASK_WORKER.value)
        manifest.inputs_add_data_type(data_type_name=DataType.PINGS.value,
                                      min_count=2)

        manifest.container_parameters_required(image="orbitalinsight/raw_foottraffic:84c76f7f",
                                               command=["python",
                                                        "/orbital/base/algorithms/"
                                                        "raw_foottraffic/src/py/raw_foottraffic/simple_foottraffic.py"])
        manifest.outputs_add_data_type(data_type_name="device_visits",
                                       observation_value_columns=["unique_device_count"])

        manifest_struct = manifest.get()
        print(manifest_struct)

    @pytest.mark.asyncio
    async def test_algorithm_manifest_imagery(self):
        manifest = AlgorithmManifest()
        manifest.metadata_required(description="Testing algo manifest builder for imagery")
        manifest.interface_required(interface_type=InterfaceType.FILESYSTEM_TASK_WORKER.value)
        manifest.inputs_add_data_type(data_type_name=DataType.OPTICAL_IMAGERY.value, parameters={"image_processing_spec": "PL-SkySatCollect"})
        manifest.container_parameters_required(image="orbitalinsight/skysat_ship_detector:1a354abc",
                                               command=["python",
                                                        "/orbital/base/algorithms/skysat_ship/src/py/skysat_ship_detector/simple_inference.py"])
        manifest.outputs_add_data_type(data_type_name="object_detection", observation_value_columns=["count"])

        manifest_json = MessageToDict(manifest.get())
        assert manifest_json["metadata"] == {"description": "Testing algo manifest builder for imagery"}
        assert manifest_json["interface"] == {"interface_type": InterfaceType.FILESYSTEM_TASK_WORKER.value}
        assert manifest_json["inputs"] == [{"data_type_name": DataType.OPTICAL_IMAGERY.value, "parameters": {"image_processing_spec": "PL-SkySatCollect"}}]
        assert manifest_json["container_parameters"]["image"] ==  "orbitalinsight/skysat_ship_detector:1a354abc"
        assert manifest_json["container_parameters"]["command"] == ["python","/orbital/base/algorithms/skysat_ship/src/py/skysat_ship_detector/simple_inference.py"]
        assert manifest_json["outputs"] == {"observation_value_columns": ["count"], "output_data_types": ["object_detection"]}



from __future__ import annotations

# ^^ postpones type checking, evaluation of annotations - does on run time NOT on import
from typing import List

from google.protobuf.struct_pb2 import Struct
from google.protobuf.timestamp_pb2 import Timestamp
from elements_api import ElementsAsyncClient
from elements_api.models.aoi_collection_pb2 import AOICollectionCreateRequest, AOICollectionCloneRequest, \
    AOICollectionGetRequest, AOIInfo, AOICollectionAddRequest, \
    AOICollectionRemoveRequest, AOICollection
from elements_api.models.aoi_pb2 import AOIUploadRequest, AOIGetRequest, AOIInput, \
    AOICreateRequest, AOIVersion, AOITransaction
from elements_api.models.aoi_version_pb2 import AOIField
from elements_api.models.aoi_version_pb2 import AOIVersionListRequest, AOIVersionCreateRequest, AOIVersionGetRequest
from elements_api.models.common_models_pb2 import Pagination


class APIAOI:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def upload(self, aoi_collection_id: str, file_path: str) -> AOITransaction:
        """
        Upload new AOIs to the system by passing in the raw bytes. Acceptable file formats are .zip (i.e. shapefile),
        geojson, kml, kmz, and wkt. The aoi_id and aoi_version of the created AOIs are returned. This call is
        asynchronous. It will return a aoi_upload_transaction_id that can be used to check the state of the upload.

        :param aoi_collection_id: The collection ID to upload AOIs to.
        :param file_path: File path to your aois
        :return: list of AOI identifiers
        """
        response = await self.__client.api.aoi.upload(self.__get_bytes_iterator(file_name=file_path,
                                                                                aoi_collection_id=aoi_collection_id))

        return response.aoi_transaction

    @staticmethod
    async def __get_bytes_iterator(file_name: str, aoi_collection_id: str):
        eof = False
        with open(file_name, 'rb') as f:
            while not eof:
                chunk = f.read(1024 * 1024)
                if chunk:
                    # Yield the new chunk of the file for the latest bytes
                    yield AOIUploadRequest(aoi_collection_id=aoi_collection_id, chunk=chunk)
                else:
                    # No more to read. End iteration.
                    eof = True

    async def create(self, aoi_collection_id: str,
                     aoi_inputs: List[AOIInput]) -> List:
        """
        Create a new AOI in the system by passing in the required components. Pass in an optional aoi_id to act
        as the parent and create an updated version of the AOI, which will copy fields from the parent unless explicitly
        overridden by the input parameters. The aoi_id and aoi_version of the created AOI are returned. This endpoint
        is currently limited to 100 AOIs or less. If you would like to upload more, refer to aoi.upload.

        :param aoi_inputs: AOIInput Object
        :param aoi_collection_id: The collection to create the AOI

        :return: aoi_identifiers: List
        """
        request = AOICreateRequest(
            aoi_collection_id=aoi_collection_id,
            aoi_inputs=aoi_inputs
        )
        response = await self.__client.api.aoi.create(request)
        return response.aoi_identifiers

    async def get(self, ids: List, verbose: bool = False) -> List[AOIVersion]:
        """
        Get the metadata about the specified AOIs. Setting the verbose flag will also include the bytes
        in wkt format describing the geometry.

        :param ids:
        :param verbose:
        :return: List[AOIVersion]
        """
        request = AOIGetRequest(
            aoi_ids=ids,
            verbose=verbose
        )
        response = await self.__client.api.aoi.get(request)
        return response.aoi_versions

    async def delete(self):
        pass


class APIAOITransaction:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client


class APIAOIVersion:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def create(self, aoi_id: str, aoi_modification_input: AOIInput) -> int:
        """
        Create a new aoi_version that is tied to the original aoi by reference of the aoi_id. After specifying the
        aoi modifications, the original is copied and modified. Further, aoi_versions are immutable.
        :param aoi_id:
        :param aoi_modification_input:
        :return: aoi_version_id: the uint32 version id
        """
        request = AOIVersionCreateRequest(
            aoi_id=aoi_id,
            aoi_modification_input=aoi_modification_input
        )
        response = await self.__client.api.aoi_version.create(request)
        return response.aoi_version_id

    async def get(self, aoi_version_ids: List[int], aoi_fields: List[AOIField]) -> List[AOIVersion]:
        # Problem is with List[AOIField] --- didn't like list of enums
        # added import annotations
        # Python version issue?
        # Either passing in not AOIField
        # async def get(self, aoi_version_ids: List[int], aoi_fields: List[int]) -> List[AOIVersion]:
        # async def get(self, aoi_version_ids: List[int], aoi_fields: List[AOIField]) -> List[AOIVersion]:
        """
        Get the actual AOI details specified by the provided version. Additionally, choose which metadata fields
        should be returned. A list of aoi_collections that they aoi_version is a part of is also returned.
        :param aoi_version_ids:
        :param aoi_fields:
        :return: List[AOIVersion]
        """
        request = AOIVersionGetRequest(
            ids=aoi_version_ids,
            aoi_fields=aoi_fields
        )
        response = await self.__client.api.aoi_version.get(request)
        return response.aoi_versions

    async def list(self, **kwargs) -> List[AOIVersion]:
        """
        List the available aois, including on the latest versions by default that match the specific filters.
        Additional options include the ability to list out all fields of the aoi_version
        by setting the verbose flag to True.

        :param kwargs
            - geom_wkt: str
            - category: str
            - tags: List
            - search_text: str
            - min_created_on: Datetime
            - max_created_on: Datetime
            - aoi_fields: AOIField
            - pagination: Pagination
            - verbose: bool

        :return:
        """
        fragments = []
        if 'pagination' not in kwargs.keys():
            fragments.append(AOIVersionListRequest(pagination=Pagination(
                page_token="1",
                page_size=1000,
                next_page_token="2"
            )))

        for key in kwargs.keys():
            if key == 'geom_wkt':
                fragments.append(AOIVersionListRequest(geom_wkt=kwargs[key]))
            if key == 'category':
                fragments.append(AOIVersionListRequest(category=kwargs[key]))
            if key == 'tags':
                fragments.append(AOIVersionListRequest(tags=kwargs[key]))
            if key == 'search_text':
                fragments.append(AOIVersionListRequest(search_text=kwargs[key]))
            if key == 'min_created_on':
                min_created_on = Timestamp()
                min_created_on.FromDatetime(kwargs[key])
                fragments.append(AOIVersionListRequest(min_created_on=min_created_on))
            if key == 'max_created_on':
                max_created_on = Timestamp()
                max_created_on.FromDatetime(kwargs[key])
                fragments.append(AOIVersionListRequest(max_created_on=max_created_on))
            if key == 'aoi_fields':
                fragments.append(AOIVersionListRequest(search_text=kwargs[key]))
            if key == 'verbose':
                fragments.append(AOIVersionListRequest(search_text=kwargs[key]))
            if key == 'pagination':
                fragments.append(AOIVersionListRequest(search_text=kwargs[key]))

        request = AOIVersionListRequest()
        for param in fragments:
            request.MergeFrom(param)

        response = await self.__client.api.aoi_version.list(request)
        return response.aoi_versions


class APIAOICollection:
    def __init__(self, client: ElementsAsyncClient, timeout):
        self.__timeout = timeout
        self.__client = client

    async def create(self, aoi_collection_name: str) -> AOICollection:
        """
        Description:
        API call to create a new AOI collection.

        :param aoi_collection_name: The human-readable name of the collection that will be associated with a UUID for
        further usages.
        :return: id
        """
        request = AOICollectionCreateRequest(
            name=aoi_collection_name
        )
        response = await self.__client.api.aoi_collection.create(
            request, timeout=self.__timeout
        )

        return response.aoi_collection

    async def get(self,
                  aoi_collection_id: str,
                  pagination: Pagination = None) -> List[AOIInfo]:
        """
        Get the metadata about the collection and the AOIs that belong to the specified AOI collection..
        Setting the verbose flag will also include the bytes in wkt format describing the geometry.
        :param pagination:
        :param aoi_collection_id: the aoi collection id

        :return: List[AOIInfo]
        """

        request = AOICollectionGetRequest(
            id=aoi_collection_id
        )
        response = await self.__client.api.aoi_collection.get(request, timeout=self.__timeout)
        return response.aoi_info

    async def list(self, **kwargs):
        """

        :param kwargs:
            - min_created_on: Datetime
            - max_created_on: Datetime
            - search_text: str
            - pagination: Pagination
        :return:
        """
        fragments = []
        if 'pagination' not in kwargs.keys():
            fragments.append(AOIVersionListRequest(pagination=Pagination(
                page_token="1",
                page_size=1000,
                next_page_token="2"
            )))
        for key in kwargs.keys():
            if key == 'search_text':
                fragments.append(AOIVersionListRequest(search_text=kwargs[key]))
            if key == 'min_created_on':
                min_created_on = Timestamp()
                min_created_on.FromDatetime(kwargs[key])
                fragments.append(AOIVersionListRequest(min_created_on=min_created_on))
            if key == 'max_created_on':
                max_created_on = Timestamp()
                max_created_on.FromDatetime(kwargs[key])
                fragments.append(AOIVersionListRequest(max_created_on=max_created_on))
            if key == 'pagination':
                fragments.append(AOIVersionListRequest(search_text=kwargs[key]))
        request = AOIVersionListRequest()
        for fragment in fragments:
            request.MergeFrom(fragment)
        response = await self.__client.api.aoi_version.list(request)
        return response

    async def add(self, id: str, aoi_version_ids: List):
        """
        Cannot add to a "locked" collection.
        :param id: The AOI Collection id to add aoi versions to.
        :param aoi_version_ids: The version IDs of to be added to the aoi collection
        :return: Empty
        """
        request = AOICollectionAddRequest(
            id=id,
            aoi_version_ids=aoi_version_ids
        )
        await self.__client.api.aoi_collection.add(request, timeout=self.__timeout)

    async def remove(self, aoi_collection_id: str, aoi_version_ids: List):
        """
        Cannot remove from a "locked" collection.
        :param aoi_collection_id:
        :param aoi_version_ids:
        :return:
        """
        request = AOICollectionRemoveRequest(
            id=aoi_collection_id,
            aoi_version_ids=aoi_version_ids
        )
        await self.__client.api.aoi_collection.remove(request, timeout=self.__timeout)

    async def delete(self, aoi_collection_ids: List):
        pass

    async def clone(self, id: str, name: str) -> str:
        """
        Description
        This will let you clone an entire AOI collection.

        :param id: The aoi collection id to clone
        :param name: The new name for the cloned cloned collection
        :return: id: str
        """
        request = AOICollectionCloneRequest(
            id=id,
            name=name
        )
        response = await self.__client.api.aoi_collection.cone(request, timeout=self.__timeout)
        return response.aoi_collection_id


class AOIInputBuilder:
    @staticmethod
    def build(**kwargs) -> AOIInput:
        aoi_input = AOIInput()
        if 'aoi_id' in kwargs.keys():
            aoi_input.MergeFrom(AOIInput(aoi_id=kwargs['aoi_id']))
        if 'geom_wkt' in kwargs.keys():
            aoi_input.MergeFrom(AOIInput(geom_wkt=kwargs['geom_wkt']))
        if 'name' in kwargs.keys():
            aoi_input.MergeFrom(AOIInput(name=kwargs['name']))
        if 'category' in kwargs.keys():
            aoi_input.MergeFrom(AOIInput(category=kwargs['category']))
        if 'type' in kwargs.keys():
            aoi_input.MergeFrom(AOIInput(type=kwargs['type']))
        if 'tags' in kwargs.keys():
            aoi_input.MergeFrom(AOIInput(tags=kwargs['tags']))
        if 'attributes' in kwargs.keys():
            attributes = Struct()
            attributes.update(kwargs['attributes'])
            aoi_input.MergeFrom(AOIInput(attributes=attributes))

        return aoi_input
